@@ -32,7 +32,7 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 		CCacheEntry &entry = *iter;
 		if (entry.listing.path != listing.path)
 			continue;
-		
+
 		entry.modificationTime = CTimeEx::Now();
 		entry.createTime = entry.modificationTime.GetTime();
 
@@ -47,7 +47,7 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 				if (parent.path == parentPath && parent.subDir == subDir)
 					return;
 			}
-	
+
 			// Store parent
 			CCacheEntry::t_parent parent;
 			parent.path = parentPath;
@@ -316,16 +316,16 @@ bool CDirectoryCache::InvalidateFile(const CServer &server, const CServerPath &p
 				entry.listing[i].unsure = true;
 				//if (entry.listing[i].name == filename)
 					//matchCase = true;
-			}				
+			}
 		}
-		entry.listing.m_hasUnsureEntries |= UNSURE_UNSURE;
+		entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
 		entry.modificationTime = CTimeEx::Now();
 	}
 
 	return true;
 }
 
-bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, enum Filetype type /*=file*/, int size /*=-1*/)
+bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, enum Filetype type /*=file*/, wxLongLong size /*=-1*/)
 {
 	CServerEntry* pServerEntry = GetServerEntry(server);
 	if (!pServerEntry)
@@ -341,16 +341,31 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 			continue;
 
 		bool matchCase = false;
-		for (unsigned int i = 0; i < entry.listing.GetCount(); i++)
+		unsigned int i;
+		for (i = 0; i < entry.listing.GetCount(); i++)
 		{
 			if (!filename.CmpNoCase(cEntry.listing[i].name))
 			{
-				if (cEntry.listing[i].name == filename)
-					matchCase = true;
 				entry.listing[i].unsure = true;
-			}				
+				if (cEntry.listing[i].name == filename)
+				{
+					matchCase = true;
+					break;
+				}
+			}
 		}
-		if (!matchCase && type != unknown && mayCreate)
+
+		if (matchCase)
+		{
+			enum Filetype old_type = entry.listing[i].dir ? dir : file;
+			if (type != old_type)
+				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_invalid;
+			else if (type == dir)
+				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_dir_changed;
+			else
+				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_file_changed;
+		}
+		else if (type != unknown && mayCreate)
 		{
 			const unsigned int count = entry.listing.GetCount();
 			entry.listing.SetCount(count + 1);
@@ -362,10 +377,21 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 			direntry.dir = (type == dir);
 			direntry.link = 0;
 			direntry.unsure = true;
-			entry.listing.m_hasUnsureEntries |= UNSURE_ADD;
+			switch (type)
+			{
+			case dir:
+				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_dir_added;
+				break;
+			case file:
+				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_file_added;
+				break;
+			default:
+				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_invalid;
+				break;
+			}
 		}
 		else
-			entry.listing.m_hasUnsureEntries |= UNSURE_CHANGE;
+			entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
 		entry.modificationTime = CTimeEx::Now();
 
 		updated = true;
@@ -392,7 +418,7 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 			if (entry.listing[i].name == filename)
 				matchCase = true;
 		}
-		
+
 		if (matchCase)
 		{
 			unsigned int i;
@@ -402,7 +428,7 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 			wxASSERT(i != entry.listing.GetCount());
 
 			CDirectoryListing& listing = iter->listing;
-			listing.RemoveEntry(i);
+			listing.RemoveEntry(i); // This does set m_hasUnsureEntries
 		}
 		else
 		{
@@ -411,9 +437,9 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 				if (!filename.CmpNoCase(entry.listing[i].name))
 				{
 					iter->listing[i].unsure = true;
-				}				
+				}
 			}
-			iter->listing.m_hasUnsureEntries |= UNSURE_CONFUSED;
+			iter->listing.m_hasUnsureEntries |= CDirectoryListing::unsure_invalid;
 		}
 		iter->modificationTime = CTimeEx::Now();
 	}
@@ -443,7 +469,7 @@ bool CDirectoryCache::GetChangeTime(CTimeEx& time, const CServer &server, const
 	for (tCacheConstIter iter = pServerEntry->cacheList.begin(); iter != pServerEntry->cacheList.end(); iter++)
 	{
 		const CCacheEntry &entry = *iter;
-		
+
 		if (entry.listing.path != path)
 			continue;
 
@@ -487,7 +513,7 @@ void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path,
 
 		newList.push_back(*iter);
 	}
-	
+
 	pServerEntry->cacheList = newList;
 
 	RemoveFile(server, path, filename);
@@ -514,7 +540,7 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 			{
 				listing[i].name = fileTo;
 				listing[i].unsure = true;
-				listing.m_hasUnsureEntries |= UNSURE_CHANGE;
+				listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
 			}
 			return;
 		}
