@@ -18,6 +18,19 @@ static const t_protocolInfo protocolInfos[] = {
 	{ UNKNOWN, _T(""),      21,  false, _T("") }
 };
 
+static const wxString typeNames[SERVERTYPE_MAX] = {
+	wxTRANSLATE("Default (Autodetect)"),
+	_T("Unix"),
+	_T("VMS"),
+	_T("DOS"),
+	_T("MVS, OS/390, z/OS"),
+	_T("VxWorks"),
+	_T("z/VM"),
+	_T("HP NonStop"),
+	wxTRANSLATE("DOS-like with virtual paths"),
+	_T("Cygwin")
+};
+
 CServer::CServer()
 {
 	Initialize();
@@ -192,7 +205,7 @@ wxString CServer::GetUser() const
 {
 	if (m_logonType == ANONYMOUS)
 		return _T("anonymous");
-	
+
 	return m_user;
 }
 
@@ -200,7 +213,7 @@ wxString CServer::GetPass() const
 {
 	if (m_logonType == ANONYMOUS)
 		return _T("anon@localhost");
-	
+
 	return m_pass;
 }
 
@@ -228,6 +241,8 @@ CServer& CServer::operator=(const CServer &op)
 	m_encodingType = op.m_encodingType;
 	m_customEncoding = op.m_customEncoding;
 	m_postLoginCommands = op.m_postLoginCommands;
+	m_bypassProxy = op.m_bypassProxy;
+	m_name = op.m_name;
 
 	return *this;
 }
@@ -275,6 +290,8 @@ bool CServer::operator==(const CServer &op) const
 	}
 	if (m_postLoginCommands != op.m_postLoginCommands)
 		return false;
+	if (m_bypassProxy != op.m_bypassProxy)
+		return false;
 
 	// Do not compare number of allowed multiple connections
 
@@ -344,7 +361,7 @@ bool CServer::operator<(const CServer &op) const
 		return true;
 	else if (m_timezoneOffset > op.m_timezoneOffset)
 		return false;
-	
+
 	if (m_pasvMode < op.m_pasvMode)
 		return true;
 	else if (m_pasvMode > op.m_pasvMode)
@@ -362,6 +379,10 @@ bool CServer::operator<(const CServer &op) const
 		else if (m_customEncoding > op.m_customEncoding)
 			return false;
 	}
+	if (m_bypassProxy < op.m_bypassProxy)
+		return true;
+	else if (m_bypassProxy > op.m_bypassProxy)
+		return false;
 
 	// Do not compare number of allowed multiple connections
 
@@ -433,7 +454,7 @@ bool CServer::SetHost(wxString host, unsigned int port)
 
 	if (m_protocol == UNKNOWN)
 		m_protocol = GetProtocolFromPort(m_port);
-	
+
 	return true;
 }
 
@@ -447,7 +468,7 @@ bool CServer::SetUser(const wxString& user, const wxString& pass /*=_T("")*/)
 
 	m_user = user;
 	m_pass = pass;
-	
+
 	return true;
 }
 
@@ -546,6 +567,7 @@ void CServer::Initialize()
 	m_maximumMultipleConnections = 0;
 	m_encodingType = ENCODING_AUTO;
 	m_customEncoding = _T("");
+	m_bypassProxy = false;
 }
 
 bool CServer::SetEncodingType(enum CharsetEncoding type, const wxString& encoding /*=_T("")*/)
@@ -566,7 +588,7 @@ bool CServer::SetCustomEncoding(const wxString& encoding)
 
 	m_encodingType = ENCODING_CUSTOM;
 	m_customEncoding = encoding;
-	
+
 	return true;
 }
 
@@ -599,7 +621,7 @@ enum ServerProtocol CServer::GetProtocolFromPort(unsigned int port, bool default
 		if (protocolInfos[i].defaultPort == port)
 			return protocolInfos[i].protocol;
 	}
-	
+
 	if (defaultOnly)
 		return UNKNOWN;
 
@@ -681,3 +703,27 @@ wxString CServer::GetPrefixFromProtocol(const enum ServerProtocol protocol)
 
 	return _T("");
 }
+
+void CServer::SetBypassProxy(bool val)
+{
+  m_bypassProxy = val;
+}
+
+bool CServer::GetBypassProxy() const
+{
+  return m_bypassProxy;
+}
+
+bool CServer::ProtocolHasDataTypeConcept(const enum ServerProtocol protocol)
+{
+	if (protocol == FTP || protocol == FTPES || protocol == FTPES)
+		return true;
+
+	return false;
+}
+
+wxString CServer::GetNameFromServerType(enum ServerType type)
+{
+	wxASSERT(type != SERVERTYPE_MAX);
+	return wxGetTranslation(typeNames[type]);
+}
