@@ -3,6 +3,32 @@
 
 #define FTP_MVS_DOUBLE_QUOTE (wxChar)0xDC
 
+struct CServerTypeTraits
+{
+	wxChar separator;
+	bool has_root; // Root = simply separator nothing else
+	wxChar left_enclosure; // Example: VMS paths: [FOO.BAR]
+	wxChar right_enclosure;
+	bool filename_inside_enclosure; // MVS
+	int prefixmode; //0 = normal prefix, 1 = suffix
+	wxChar separatorEscape;
+	bool has_dots; // Special meaning for .. (parent) and . (self)
+	bool separator_after_prefix;
+};
+
+static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
+	{ '/',  true,  0,    0,    false, 0, 0,   true,  false }, // Failsafe
+	{ '/',  true,  0,    0,    false, 0, 0,   true,  false },
+	{ '.',  false, '[',  ']',  false, 0, '^', false, false },
+	{ '\\', false, 0,    0,    false, 0, 0,   true,  false },
+	{ '.',  false, '\'', '\'', true,  1, 0,   false, false },
+	{ '/',  true,  0,    0,    false, 0, 0,   true,  false },
+	{ '/',  true,  0,    0,    false, 0, 0,   true,  false }, // Same as Unix
+	{ '.',  false, 0,    0,    false, 0, 0,   false, false },
+	{ '\\', true,  0,    0,    false, 0, 0,   true,  false },
+	{ '/',  true,  0,    0,    false, 0, 0,   true,  true  } // Cygwin is like Unix but has optional prefix of form "//server"
+};
+
 CServerPath::CServerPath()
 {
 	m_type = DEFAULT;
@@ -18,7 +44,7 @@ CServerPath::CServerPath(const CServerPath &path, wxString subdir /*=_T("")*/)
 
 	subdir.Trim(true);
 	subdir.Trim(false);
-	
+
 	if (subdir == _T(""))
 		return;
 
@@ -52,9 +78,6 @@ bool CServerPath::SetPath(wxString newPath)
 
 bool CServerPath::SetPath(wxString &newPath, bool isFile)
 {
-	m_Segments.clear();
-	m_prefix = _T("");
-
 	wxString path = newPath;
 	wxString file;
 
@@ -62,12 +85,7 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	path.Trim(false);
 
 	if (path == _T(""))
-	{
-		m_bEmpty = true;
 		return false;
-	}
-	else
-		m_bEmpty = false;
 
 	if (m_type == DEFAULT)
 	{
@@ -92,246 +110,22 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 			if (slash == -1 || slash > pos1)
 				m_type = VXWORKS;
 		}
-		
+		else if (path[0] == '\\')
+			m_type = DOS_VIRTUAL;
+
 		if (m_type == DEFAULT)
 			m_type = UNIX;
 	}
 
-	int pos;
-	switch (m_type)
-	{
-	case VMS:
-		if (isFile)
-		{
-			pos = path.Find(']', true);
-			if (pos == -1)
-			{
-				m_bEmpty = true;
-				return false;
-			}
-			file = path.Mid(pos + 1);
-			path = path.Left(pos + 1);
-		}
-		pos = path.Find(_T("["));
-		if (pos == -1 || path.Right(1) != _T("]"))
-		{
-			m_bEmpty = true;
-			return false;
-		}
-		else
-		{
-			path.RemoveLast();
-			if (pos)
-				m_prefix = path.Left(pos);
-			path = path.Mid(pos + 1);
-			pos = path.Find(_T("."));
-			bool append = false;
-			while (pos != -1)
-			{
-				bool wasAppend = append;
-				wxString segment = path.Left(pos);
-				if (segment.Last() == '^')
-				{
-					append = true;
-					segment.RemoveLast();
-					segment += _T(".");
-				}
-				else
-					append = false;
+	m_Segments.clear();
+	m_bEmpty = true;
+	m_prefix = _T("");
 
-				if (wasAppend)
-					m_Segments.back() += segment;
-				else
-					m_Segments.push_back(segment);					
-
-				path = path.Mid(pos + 1);
-				pos = path.Find(_T("."));
-			}
-			if (path != _T(""))
-			{
-				if (append)
-					m_Segments.back() += path;
-				else
-					m_Segments.push_back(path);
-			}
-		}
-		break;
-	case MVS:
-		{
-			int i = 0;
-			wxChar c = path.c_str()[i];
-			while (c == FTP_MVS_DOUBLE_QUOTE || c == '\'' || c == '.')
-				c = path.c_str()[++i];
-			path.Remove(0, i);
-			
-			while (path != _T(""))
-			{
-				c = path.Last();
-				if (c != FTP_MVS_DOUBLE_QUOTE && c != '\'')
-					break;
-				else
-					path.RemoveLast();
-			}
-
-			while (path.Replace(_T(".."), _T(".")));
-
-			int pos = path.Find(_T("."));
-			while (pos != -1)
-			{
-				m_Segments.push_back(path.Left(pos));
-				path = path.Mid(pos + 1);
-				pos = path.Find( _T(".") );
-			}
-			if (path != _T(""))
-				m_Segments.push_back(path);
-			else
-				m_prefix = _T(".");
-
-			if (isFile)
-			{
-				if (m_prefix == _T("."))
-					return false;
-
-				if (m_Segments.empty())
-					return false;
-				file = m_Segments.back();
-				m_Segments.pop_back();
-
-				int pos = file.Find('(');
-				int pos2 = file.Find(')');
-				if (pos != -1)
-				{
-					if (!pos || pos2 <= pos || pos2 != (int)file.Length() - 2)
-						return false;
-					m_Segments.push_back(file.Left(pos));
-					file = file.Mid(pos + 1, pos2 - pos - 1);
-					m_prefix = _T("");
-				}
-				else if (pos2 != -1)
-					return false;
-				else
-					m_prefix = _T(".");
-			}
-		}
-		break;
-	case VXWORKS:
-		{
-			int colon2;
-			if (path[0] != ':' || (colon2 = path.Mid(1).Find(':')) < 1)
-			{
-				m_bEmpty = true;
-				return false;
-			}
-			int slash = path.Find('/');
-			if (slash != -1 && slash <= colon2)
-			{
-				m_bEmpty = true;
-				return false;
-			}
-			
-			m_prefix = path.Left(colon2 + 2);
-			path = path.Mid(colon2 + 1);
-			goto set_path_default;
-		}
-		break;
-	case HPNONSTOP:
-		// Paths of the form \mysys.$myvol.mysubvol
-		// Files of the form \mysys.$myvol.mysubvol.file
-		if (path[0] != '\\')
-		{
-			m_bEmpty = true;
-			return false;
-		}
-		while (true)
-		{
-			int pos = path.Find('.');
-			if (!pos || pos == (int)path.Len() - 1)
-			{
-				m_bEmpty = true;
-				return false;
-			}
-			if (pos == -1)
-			{
-				if (isFile)
-				{
-					if (m_Segments.empty())
-					{
-						m_bEmpty = true;
-						return false;
-					}
-					newPath = path;
-					return true;
-				}
-				m_Segments.push_back(path);
-				return true;				
-			}
-	
-			m_Segments.push_back(path.Left(pos));
-			path = path.Mid(pos + 1);
-		}
-		break;
-	case DOS:
-		// Check for starting drive letter
-		path.Replace(_T("\\"), _T("/"));
-		pos = path.Find('/');
-		if (pos != 2 || path.c_str()[1] != ':' ||
-			!((path.c_str()[0] >= 'A' && path.c_str()[0] <= 'Z') || (path.c_str()[0] >= 'a' && path.c_str()[0] <= 'z')))
-		{
-			m_bEmpty = true;
-			return false;
-		}
-		// No break on purpose!
-	default:
-set_path_default:
-		while (path.Replace(_T("//"), _T("/")));
-
-		if (path.c_str()[0] == '/')
-			path.Remove(0, 1);
-
-		if (isFile)
-		{
-			pos = path.Find('/', true);
-			if (pos == -1 || static_cast<size_t>(pos) == (path.Length() - 1))
-			{
-				m_bEmpty = true;
-				return false;
-			}
-			file = path.Mid(pos + 1);
-			path = path.Left(pos + 1);
-
-			if (file == _T(".") || file == _T(".."))
-			{
-				m_bEmpty = true;
-				return false;
-			}
-		}
-		else if (path != _T("") && path.Right(1) != _T("/"))
-			path = path + _T("/");
-
-		pos = path.Find(_T("/"));
-		while (pos != -1)
-		{
-			wxString segment = path.Left(pos);
-			if (segment == _T(".."))
-			{
-				if (m_Segments.empty())
-				{
-					m_bEmpty = true;
-					return false;
-				}
-				m_Segments.pop_back();
-			}
-			else if (segment != _T("."))
-				m_Segments.push_back(segment);
-			path = path.Mid(pos + 1);
-			pos = path.Find(_T("/"));
-		}
-
-		break;
-	}
+	if (!ChangePath(path, isFile))
+		return false;
 
 	if (isFile)
-		newPath = file;
+		newPath = path;
 	return true;
 }
 
@@ -342,62 +136,46 @@ wxString CServerPath::GetPath() const
 
 	wxString path;
 
-	switch (m_type)
-	{
-	case VMS:
-		{
-			path = m_prefix + _T("[");
-			for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-			{
-				wxString segment = *iter;
-				segment.Replace(_T("."), _T("^."));
-				path += segment + _T(".");
-			}
-			path.RemoveLast();
-			path += _T("]");
-		}
-		break;
-	case DOS:
-		{
-			for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-				path += *iter + _T("\\");
-		}
-		break;
-	case MVS:
-		path = _T("'");
-		for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-		{
-			if (iter != m_Segments.begin())
-				path += _T(".");
-			path += *iter;
-		}
-		path += m_prefix + _T("'");
-		break;
-	case VXWORKS:
+	if (!traits[m_type].prefixmode)
 		path = m_prefix;
-		for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-		{
-			if (iter != m_Segments.begin())
-				path += _T("/");
-			path += *iter;
-		}
-		break;
-	case HPNONSTOP:
-		for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-		{
-			if (iter != m_Segments.begin())
-				path += _T(".");
-			path += *iter;
-		}
-		break;
-	default:
-		path = _T("/");
-		for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-			path += *iter + _T("/");
 
-		break;
+	if (traits[m_type].left_enclosure != 0)
+		path += traits[m_type].left_enclosure;
+	if (m_Segments.empty() && (!traits[m_type].has_root || m_prefix.empty() || traits[m_type].separator_after_prefix))
+		path += traits[m_type].separator;
+
+	for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
+	{
+		const wxString& segment = *iter;
+		if (iter != m_Segments.begin())
+			path += traits[m_type].separator;
+		else if (traits[m_type].has_root)
+		{
+			if (m_prefix.empty() || traits[m_type].separator_after_prefix)
+				path += traits[m_type].separator;
+		}
+
+		if (traits[m_type].separatorEscape)
+		{
+			wxString tmp = segment;
+			tmp.Replace((wxString)traits[m_type].separator, (wxString)traits[m_type].separatorEscape + traits[m_type].separator);
+			path += tmp;
+		}
+		else
+			path += segment;
 	}
 
+	if (traits[m_type].prefixmode)
+		path += m_prefix;
+
+	if (traits[m_type].right_enclosure != 0)
+		path += traits[m_type].right_enclosure;
+
+	// DOS is strange.
+	// C: is current working dir on drive C, C:\ the drive root.
+	if (m_type == DOS && m_Segments.size() == 1)
+		path += traits[m_type].separator;
+
 	return path;
 }
 
@@ -406,7 +184,7 @@ bool CServerPath::HasParent() const
 	if (m_bEmpty)
 		return false;
 
-	if (m_type == DOS || m_type == VMS || m_type == MVS || m_type == HPNONSTOP)
+	if (!traits[m_type].has_root)
 		return m_Segments.size() > 1;
 
 	return !m_Segments.empty();
@@ -493,7 +271,7 @@ bool CServerPath::SetSafePath(wxString path)
 	path = path.Mid(pos + 1);
 	if (path.Length() < len)
 		return false;
-	
+
 	if (len)
 	{
 		m_prefix = path.Left(len);
@@ -510,7 +288,7 @@ bool CServerPath::SetSafePath(wxString path)
 			path = path.Mid(1);
 			continue;
 		}
-		
+
 		if (!path.Left(pos).ToULong(&len) || !len)
 			return false;
 		path = path.Mid(pos + 1);
@@ -520,7 +298,7 @@ bool CServerPath::SetSafePath(wxString path)
 		m_Segments.push_back(path.Left(len));
 		path = path.Mid(len + 1);
 	}
-	
+
 	m_bEmpty = false;
 
 	return true;
@@ -546,17 +324,25 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 	if (m_bEmpty || path.m_bEmpty)
 		return false;
 
-	if (cmpNoCase && m_prefix.CmpNoCase(path.m_prefix))
-		return false;
-	if (!cmpNoCase && m_prefix != path.m_prefix)
-		return false;
-
 	if (m_type != path.m_type)
 		return false;
 
 	if (!HasParent())
 		return false;
 
+	if (traits[m_type].prefixmode != 1)
+	{
+		if (cmpNoCase && m_prefix.CmpNoCase(path.m_prefix))
+			return false;
+		if (!cmpNoCase && m_prefix != path.m_prefix)
+			return false;
+	}
+
+	// On MVS, dirs like 'FOO.BAR' without trailing dot cannot have
+	// subdirectories
+	if (traits[m_type].prefixmode == 1 && path.m_prefix == _T(""))
+		return false;
+
 	tConstSegmentIter iter1 = m_Segments.begin();
 	tConstSegmentIter iter2 = path.m_Segments.begin();
 	while (iter1 != m_Segments.end())
@@ -599,7 +385,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 	dir.Trim(true);
 	dir.Trim(false);
-	
+
 	if (dir == _T(""))
 	{
 		if (IsEmpty() || isFile)
@@ -612,10 +398,10 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 	{
 	case VMS:
 		{
-			int pos1 = subdir.Find(_T("["));
+			int pos1 = dir.Find(traits[m_type].left_enclosure);
 			if (pos1 == -1)
 			{
-				int pos2 = dir.Find(']', true);
+				int pos2 = dir.Find(traits[m_type].right_enclosure, true);
 				if (pos2 != -1)
 					return false;
 
@@ -624,246 +410,149 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 					if (IsEmpty())
 						return false;
 
-					subdir = dir;
-					return true;
+					file = dir;
+					break;
 				}
-
-				while (dir.Replace(_T(".."), _T(".")));
 			}
 			else
 			{
-				int pos2 = dir.Find(']', true);
-				if (pos2 == -1)
+				int pos2 = dir.Find(traits[m_type].right_enclosure, true);
+				if (pos2 == -1 || pos2 <= pos1 + 1)
 					return false;
 
-				if (isFile && static_cast<size_t>(pos2) == (dir.Length() - 1))
-					return false;
-				if (isFile && static_cast<size_t>(pos2) != (dir.Length() - 1))
-					return false;
-				if (pos2 <= pos1)
+				bool enclosure_is_last = static_cast<size_t>(pos2) == (dir.Length() - 1);
+				if (isFile == enclosure_is_last)
 					return false;
 
 				if (isFile)
 					file = dir.Mid(pos2 + 1);
 				dir = dir.Left(pos2);
-				
+
 				if (pos1)
 					m_prefix = dir.Left(pos1);
-				else
-					m_prefix = _T("");
 				dir = dir.Mid(pos1 + 1);
 
 				m_Segments.clear();
 			}
-			int pos = dir.Find(_T("."));
-			bool append = false;
-			while (pos != -1)
-			{
-				bool wasAppend = append;
-				wxString segment = dir.Left(pos);
-				if (segment.Last() == '^')
-				{
-					append = true;
-					segment.RemoveLast();
-					segment += _T(".");
-				}
-				else
-					append = false;
 
-				if (wasAppend)
-					m_Segments.back() += segment;
-				else
-					m_Segments.push_back(segment);					
-
-				dir = dir.Mid(pos + 1);
-				pos = dir.Find(_T("."));
-			}
-			if (dir != _T(""))
-			{
-				if (append)
-					m_Segments.back() += dir;
-				else
-					m_Segments.push_back(dir);
-			}
+			if (!Segmentize(dir, m_Segments))
+				return false;
+			if (m_Segments.empty() && m_bEmpty)
+				return false;
 		}
 		break;
 	case DOS:
 		{
-			dir.Replace(_T("\\"), _T("/"));
-			while (dir.Replace(_T("//"), _T("/")));
 			if (dir.Length() >= 2 && dir.c_str()[1] == ':')
 				m_Segments.clear();
-			else if (dir.Left(1) == _T("/"))
+			else if (dir[0] == traits[m_type].separator)
 			{
 				if (m_Segments.empty())
-				{
-					Clear();
 					return false;
-				}
 				wxString first = m_Segments.front();
 				m_Segments.clear();
 				m_Segments.push_back(first);
 				dir = dir.Mid(1);
 			}
-			
-			if (isFile)
-			{
-				int pos = dir.Find('/', true);
-				if (pos == (int)dir.Length() - 1)
-				{
-					Clear();
-					return false;
-				}
-				if (pos == -1)
-				{
-					subdir = dir;
-					return true;
-				}
-				else
-				{
-					file = dir.Mid(pos + 1);
-					dir = dir.Left(pos + 1);
-				}
-			}
-			else if (dir != _T("") && dir.Right(1) != _T("/"))
-				dir += _T("/");
 
-			int pos = dir.Find(_T("/"));
-			while (pos != -1)
-			{
-				wxString segment = dir.Left(pos);
-				if (segment == _T(".."))
-				{
-					if (m_Segments.size() <= 1)
-					{
-						Clear();
-						return false;
-					}
-					m_Segments.pop_back();
-				}
-				else if (segment != _T("."))
-					m_Segments.push_back(segment);
-				dir = dir.Mid(pos + 1);
-				pos = dir.Find(_T("/"));
-			}
+			if (isFile && !ExtractFile(dir, file))
+				return false;
+
+			if (!Segmentize(dir, m_Segments))
+				return false;
+			if (m_Segments.empty() && m_bEmpty)
+				return false;
 		}
 		break;
 	case MVS:
 		{
+			// Remove the double quoation some servers send in PWD reply
 			int i = 0;
-			wxChar c = subdir.c_str()[i];
+			wxChar c = dir.c_str()[i];
 			while (c == FTP_MVS_DOUBLE_QUOTE)
-				c = subdir.c_str()[++i];
-			subdir.Remove(0, i);
-			
-			while (subdir != _T(""))
+				c = dir.c_str()[++i];
+			dir.Remove(0, i);
+
+			while (dir != _T(""))
 			{
-				c = subdir.Last();
+				c = dir.Last();
 				if (c != FTP_MVS_DOUBLE_QUOTE)
 					break;
 				else
-					subdir.RemoveLast();
+					dir.RemoveLast();
 			}
 		}
-		if (subdir == _T(""))
+		if (dir == _T(""))
 			return false;
 
-		while (subdir.Replace(_T(".."), _T(".")));
-		
-		if (subdir.c_str()[0] == '\'')
+		if (dir.c_str()[0] == traits[m_type].left_enclosure)
 		{
-			if (subdir.Last() != '\'')
+			if (dir.Last() != traits[m_type].right_enclosure)
 				return false;
 
-			if (SetPath(subdir, isFile))
-				file = subdir;
-			else
-				return false;
+			dir.RemoveLast();
+			dir = dir.Mid(1);
+
+			m_Segments.clear();
 		}
-		else if (subdir.Last() == '\'')
+		else if (dir.Last() == traits[m_type].right_enclosure)
 			return false;
-		else if (!IsEmpty())
+		else if (m_bEmpty)
+			return false;
+
+		if (dir.Last() == ')')
 		{
-			if (m_prefix != _T("."))
+			// Partitioned dataset member
+			if (!isFile)
 				return false;
 
-			if (subdir.c_str()[0] == '.')
-				subdir.Remove(0, 1);
-			
-			int pos = subdir.Find('.');
-			while (pos != -1)
+			int pos = dir.Find('(');
+			if (pos == -1)
+				return false;
+			dir.RemoveLast();
+			file = dir.Mid(pos + 1);
+			dir = dir.Left(pos);
+
+			if (!m_bEmpty && m_prefix == _T("") && !dir.empty())
+				return false;
+
+			m_prefix.clear();
+		}
+		else
+		{
+			if (!m_bEmpty && m_prefix == _T(""))
 			{
-				m_Segments.push_back(subdir.Left(pos));
-				subdir = subdir.Mid(pos + 1);
+				if (dir.Find('.') != -1 || !isFile)
+					return false;
 			}
-			if (subdir != _T(""))
-			{
-				m_Segments.push_back(subdir);
-				m_prefix = _T("");
-			}
-			else
-				m_prefix = _T(".");
 
 			if (isFile)
 			{
-				if (m_prefix == _T("."))
+				if (!ExtractFile(dir, file))
 					return false;
-
-				if (m_Segments.empty())
-					return false;
-				file = m_Segments.back();
-				m_Segments.pop_back();
-
-				int pos = file.Find('(');
-				int pos2 = file.Find(')');
-				if (pos != -1)
-				{
-					if (!pos || pos2 <= pos || pos2 != (int)file.Length() - 2)
-						return false;
-					m_Segments.push_back(file.Left(pos));
-					file = file.Mid(pos + 1, pos2 - pos - 1);
-				}
-				else if (pos2 != -1)
-					return false;
-				else
-					m_prefix = _T(".");
+				m_prefix = _T(".");
 			}
+			else if (dir.Last() == '.')
+				m_prefix = _T(".");
+			else
+				m_prefix.clear();
 		}
-		else if (SetPath(subdir, isFile))
-			file = subdir;
-		else
+
+		if (!Segmentize(dir, m_Segments))
 			return false;
 		break;
 	case HPNONSTOP:
 		if (dir[0] == '\\')
 			m_Segments.clear();
-		while (true)
-		{
-			int pos = dir.Find('.');
-			if (!pos || pos == (int)dir.Len() - 1)
-			{
-				m_bEmpty = true;
-				return false;
-			}
-			if (pos == -1)
-			{
-				if (isFile)
-				{
-					if (m_Segments.empty())
-					{
-						m_bEmpty = true;
-						return false;
-					}
-					subdir = dir;
-					return true;
-				}
-				m_Segments.push_back(dir);
-				return true;				
-			}
-	
-			m_Segments.push_back(dir.Left(pos));
-			dir = dir.Mid(pos + 1);
-		}
+
+		if (isFile && !ExtractFile(dir, file))
+			return false;
+
+		if (!Segmentize(dir, m_Segments))
+			return false;
+		if (m_Segments.empty() && m_bEmpty)
+			return false;
+
 		break;
 	case VXWORKS:
 		{
@@ -877,84 +566,72 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				int colon2;
 				if ((colon2 = dir.Mid(1).Find(':')) < 1)
 					return false;
-
-				int slash = dir.Find('/');
-				if (slash != -1 && slash <= colon2)
-					return false;
-
 				m_prefix = dir.Left(colon2 + 2);
-				dir = dir.Mid(colon2 + 1);
-
-				if (dir[0] == '/')
-					return false;
+				dir = dir.Mid(colon2 + 2);
 
 				m_Segments.clear();
-
 			}
-			if (isFile && dir.Find('/') == -1)
+
+			if (isFile && !ExtractFile(dir, file))
+				return false;
+
+			if (!Segmentize(dir, m_Segments))
 				return false;
 		}
-		// No break on purpose
-	default:
+		break;
+	case CYGWIN:
 		{
-			while (dir.Replace(_T("//"), _T("/")));
-			if (dir.c_str()[0] == '/')
+			if (dir[0] == traits[m_type].separator)
 			{
-				m_prefix = _T("");
 				m_Segments.clear();
+				m_prefix.clear();
+			}
+			else if (m_bEmpty)
+				return false;
+			if (dir.Left(2) == _T("//"))
+			{
+				m_prefix = traits[m_type].separator;
 				dir = dir.Mid(1);
 			}
-			
-			if (isFile)
-			{
-				int pos = dir.Find('/', true);
-				if (pos == (int)dir.Length() - 1)
-				{
-					Clear();
-					return false;
-				}
-				if (pos == -1)
-				{
-					subdir = dir;
-					return true;
-				}
-				else
-				{
-					file = dir.Mid(pos + 1);
-					dir = dir.Left(pos + 1);
-				}
-			}
-			else if (dir != _T("") && dir.Right(1) != _T("/"))
-				dir += _T("/");
-			
-			int pos = dir.Find(_T("/"));
-			while (pos != -1)
-			{
-				wxString segment = dir.Left(pos);
-				if (segment == _T(".."))
-				{
-					if (m_Segments.empty())
-					{
-						Clear();
-						return false;
-					}
-					m_Segments.pop_back();
-				}
-				else if (segment != _T("."))
-					m_Segments.push_back(segment);
-				dir = dir.Mid(pos + 1);
-				pos = dir.Find(_T("/"));
-			}
+
+			if (isFile && !ExtractFile(dir, file))
+				return false;
+
+			if (!Segmentize(dir, m_Segments))
+				return false;
+		}
+		break;
+	default:
+		{
+			if (dir[0] == traits[m_type].separator)
+				m_Segments.clear();
+			else if (m_bEmpty)
+				return false;
+
+			if (isFile && !ExtractFile(dir, file))
+				return false;
+
+			if (!Segmentize(dir, m_Segments))
+				return false;
 		}
 		break;
 	}
 
+	if (!traits[m_type].has_root && m_Segments.empty())
+		return false;
+
 	if (isFile)
+	{
+		if (traits[m_type].has_dots)
+		{
+			if (file == _T("..") || file == _T("."))
+				return false;
+		}
 		subdir = file;
+	}
 
 	m_bEmpty = false;
 	return true;
-
 }
 
 bool CServerPath::operator==(const CServerPath &op) const
@@ -994,7 +671,7 @@ bool CServerPath::operator<(const CServerPath &op) const
 		return true;
 	if (cmp > 0)
 		return false;
-	
+
 	if (m_type > op.m_type)
 		return false;
 	else if (m_type < op.m_type)
@@ -1024,45 +701,40 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 	if (filename == _T(""))
 		return _T("");
 
-	tConstSegmentIter iter;
+	if (m_bEmpty)
+		return _T("");
+
+	if (omitPath && (!traits[m_type].prefixmode || m_prefix == _T(".")))
+		return filename;
+
+	wxString result = GetPath();
+	if (traits[m_type].left_enclosure && traits[m_type].filename_inside_enclosure)
+		result.RemoveLast();
+
 	switch (m_type)
 	{
-	case MVS:
-		if (m_prefix == _T(".") && omitPath)
-			return filename;
-
-		{
-			wxString fullpath = _T("'");
-			for (iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-				fullpath += *iter + _T(".");
-			if (m_prefix != _T("."))
-			{
-				if (fullpath.Last() == '.')
-					fullpath.RemoveLast();
-				fullpath += _T("(") + filename + _T(")");
-			}
-			else
-				fullpath += filename;
-			return fullpath + _T("'");
-		}
-		break;
-	case VXWORKS:
-		if (omitPath)
-			return filename;
-		else
-			return GetPath() + _T("/") + filename;
-	case HPNONSTOP:
-		if (omitPath)
-			return filename;
-		else
-			return GetPath() + _T(".") + filename;
-	default:
-		if (omitPath)
-			return filename;
-		else
-			return GetPath() + filename;
+		case VXWORKS:
+			if (result.Last() != traits[m_type].separator && !m_Segments.empty())
+				result += traits[m_type].separator;
+			break;
+		case VMS:
+		case MVS:
+			break;
+		default:
+			if (result.Last() != traits[m_type].separator)
+				result += traits[m_type].separator;
+			break;
 	}
-	return _T("");
+
+	if (traits[m_type].prefixmode == 1 && m_prefix.empty())
+		result += _T("(") + filename + _T(")");
+	else
+		result += filename;
+
+	if (traits[m_type].left_enclosure && traits[m_type].filename_inside_enclosure)
+		result += traits[m_type].right_enclosure;
+
+	return result;
 }
 
 int CServerPath::CmpNoCase(const CServerPath &op) const
@@ -1087,7 +759,7 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 		if (res)
 			return res;
 	}
-	
+
 	return 0;
 }
 
@@ -1104,27 +776,60 @@ bool CServerPath::AddSegment(const wxString& segment)
 
 CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 {
+	if (*this == path)
+		return *this;
+
 	if (m_bEmpty || path.m_bEmpty)
 		return CServerPath();
 
 	if (m_type != path.m_type ||
-		m_prefix != path.m_prefix)
+		(!traits[m_type].prefixmode && m_prefix != path.m_prefix))
+	{
 		return CServerPath();
+	}
 
-	if (!HasParent() || !path.HasParent())
-		return CServerPath();
+	if (!HasParent())
+	{
+		if (path.IsSubdirOf(*this, false))
+			return *this;
+		else
+			return CServerPath();
+	}
+	else if (!path.HasParent())
+	{
+		if (IsSubdirOf(path, false))
+			return path;
+		else
+			return CServerPath();
+	}
 
 	CServerPath parent;
 	parent.m_bEmpty = false;
 	parent.m_type = m_type;
 	parent.m_prefix = m_prefix;
 
+	std::list<wxString>::const_iterator last = m_Segments.end();
+	std::list<wxString>::const_iterator last2 = path.m_Segments.end();
+	if (traits[m_type].prefixmode == 1)
+	{
+		if (m_prefix.empty())
+			last--;
+		if (path.m_prefix.empty())
+			last2--;
+		parent.m_prefix = GetParent().m_prefix;
+	}
+
 	std::list<wxString>::const_iterator iter = m_Segments.begin();
 	std::list<wxString>::const_iterator iter2 = path.m_Segments.begin();
-	while (iter != m_Segments.end() && iter2 != path.m_Segments.end())
+	while (iter != last && iter2 != last2)
 	{
 		if (*iter != *iter2)
-			return parent;
+		{
+			if (!traits[m_type].has_root && parent.m_Segments.empty())
+				return CServerPath();
+			else
+				return parent;
+		}
 
 		parent.m_Segments.push_back(*iter);
 
@@ -1137,11 +842,90 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 
 wxString CServerPath::FormatSubdir(const wxString &subdir) const
 {
-	if (m_type != VMS)
+	if (!traits[m_type].separatorEscape)
 		return subdir;
 
 	wxString res = subdir;
-	res.Replace(_T("."), _T("^."));
+	res.Replace((wxString)traits[m_type].separator, (wxString)traits[m_type].separatorEscape + traits[m_type].separator);
 
 	return res;
 }
+
+bool CServerPath::Segmentize(wxString str, tSegmentList& segments)
+{
+	bool append = false;
+	while (!str.empty())
+	{
+		wxString segment;
+		int pos = str.Find(traits[m_type].separator);
+		if (pos == -1)
+		{
+			segment = str,
+			str.clear();
+		}
+		else if (!pos)
+		{
+			str = str.Mid(1);
+			continue;
+		}
+		else
+		{
+			segment = str.Left(pos);
+			str = str.Mid(pos + 1);
+		}
+
+		if (traits[m_type].has_dots)
+		{
+			if (segment == _T("."))
+				continue;
+			else if (segment == _T(".."))
+			{
+				if (segments.empty())
+					return false;
+				else
+				{
+					segments.pop_back();
+					continue;
+				}
+			}
+		}
+
+		bool append_next = false;
+		if (traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape)
+		{
+			append_next = true;
+			segment.RemoveLast();
+			segment += traits[m_type].separator;
+		}
+
+		if (append)
+			segments.back() += segment;
+		else
+			segments.push_back(segment);
+
+		append = append_next;
+	}
+	if (append)
+		return false;
+
+	return true;
+}
+
+bool CServerPath::ExtractFile(wxString& dir, wxString& file)
+{
+	int pos = dir.Find(traits[m_type].separator, true);
+	if (pos == (int)dir.Length() - 1)
+		return false;
+
+	if (pos == -1)
+	{
+		file = dir;
+		dir.clear();
+		return true;
+	}
+
+	file = dir.Mid(pos + 1);
+	dir = dir.Left(pos + 1);
+
+	return true;
+}
