@@ -5,6 +5,8 @@
 #include "directorycache.h"
 #include "directorylistingparser.h"
 #include "pathcache.h"
+#include "servercapabilities.h"
+#include <wx/tokenzr.h>
 
 class CSftpFileTransferOpData : public CFileTransferOpData
 {
@@ -19,7 +21,9 @@ enum filetransferStates
 	filetransfer_init = 0,
 	filetransfer_waitcwd,
 	filetransfer_waitlist,
-	filetransfer_transfer
+	filetransfer_mtime,
+	filetransfer_transfer,
+	filetransfer_chmtime
 };
 
 struct sftp_message
@@ -164,6 +168,13 @@ protected:
 			case sftpError:
 			case sftpVerbose:
 			case sftpStatus:
+			case sftpKexAlgorithm:
+			case sftpKexHash:
+			case sftpCipherClientToServer:
+			case sftpCipherServerToClient:
+			case sftpMacClientToServer:
+			case sftpMacServerToClient:
+			case sftpHostkey:
 				{
 					sftp_message* message = new sftp_message;
 					message->type = (sftpEventTypes)eventType;
@@ -194,7 +205,7 @@ protected:
 						if (error)
 							goto loopexit;
 
-						m_pOwner->SendRequest(new CHostKeyNotification(line.Mid(1), port, fingerprint, requestType == sftpReqHostkeyChanged));
+						m_pOwner->SendAsyncRequest(new CHostKeyNotification(line.Mid(1), port, fingerprint, requestType == sftpReqHostkeyChanged));
 					}
 					else if (requestType == sftpReqPassword)
 					{
@@ -297,25 +308,34 @@ CSftpControlSocket::~CSftpControlSocket()
 	DoClose();
 }
 
+enum connectStates
+{
+	connect_init,
+	connect_keys,
+	connect_open
+};
+
 class CSftpConnectOpData : public COpData
 {
 public:
 	CSftpConnectOpData()
 		: COpData(cmd_connect)
 	{
-		gotInitialReply = false;
 		pLastChallenge = 0;
 		criticalFailure = false;
+		pKeyFiles = 0;
 	}
 
 	virtual ~CSftpConnectOpData()
 	{
+		delete pKeyFiles;
 		delete pLastChallenge;
 	}
 
 	wxString *pLastChallenge;
-	bool gotInitialReply;
 	bool criticalFailure;
+
+	wxStringTokenizer* pKeyFiles;
 };
 
 int CSftpControlSocket::Connect(const CServer &server)
@@ -323,6 +343,8 @@ int CSftpControlSocket::Connect(const CServer &server)
 	LogMessage(Status, _("Connecting to %s:%d..."), server.GetHost().c_str(), server.GetPort());
 	SetWait(true);
 
+	m_sftpEncryptionDetails = CSftpEncryptionNotification();
+
 	delete m_pCSConv;
 	if (server.GetEncodingType() == ENCODING_CUSTOM)
 	{
@@ -340,7 +362,16 @@ int CSftpControlSocket::Connect(const CServer &server)
 		delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
 
-	m_pCurOpData = new CSftpConnectOpData;
+	CSftpConnectOpData* pData = new CSftpConnectOpData;
+	m_pCurOpData = pData;
+
+	pData->opState = connect_init;
+
+	wxStringTokenizer* pTokenizer = new wxStringTokenizer(m_pEngine->GetOptions()->GetOption(OPTION_SFTP_KEYFILES), _T("\n"), wxTOKEN_DEFAULT);
+	if (!pTokenizer->HasMoreTokens())
+		delete pTokenizer;
+	else
+		pData->pKeyFiles = pTokenizer;
 
 	m_pProcess = new wxProcess(this);
 	m_pProcess->Redirect();
@@ -381,14 +412,14 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 
 	if (!successful)
 	{
-		ResetOperation(FZ_REPLY_ERROR);
+		DoClose(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (!m_pCurOpData)
 	{
 		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
-		ResetOperation(FZ_REPLY_INTERNALERROR);
+		DoClose(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
@@ -396,19 +427,68 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 	if (!pData)
 	{
 		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("m_pCurOpData of wrong type"));
-		ResetOperation(FZ_REPLY_INTERNALERROR);
+		DoClose(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (pData->gotInitialReply)
+	switch (pData->opState)
 	{
+	case connect_init:
+		if (pData->pKeyFiles)
+			pData->opState = connect_keys;
+		else
+			pData->opState = connect_open;
+		break;
+	case connect_keys:
+		wxASSERT(pData->pKeyFiles);
+		if (!pData->pKeyFiles->HasMoreTokens())
+			pData->opState = connect_open;
+		break;
+	case connect_open:
+		m_pEngine->AddNotification(new CSftpEncryptionNotification(m_sftpEncryptionDetails));
 		ResetOperation(FZ_REPLY_OK);
 		return FZ_REPLY_OK;
+	default:
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown op state: %d"), pData->opState);
+        DoClose(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
 	}
 
-	pData->gotInitialReply = true;
+	return SendNextCommand();
+}
 
-	bool res = Send(wxString::Format(_T("open \"%s@%s\" %d"), m_pCurrentServer->GetUser().c_str(), m_pCurrentServer->GetHost().c_str(), m_pCurrentServer->GetPort()));
+int CSftpControlSocket::ConnectSend()
+{
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ConnectSend()"));
+	if (!m_pCurOpData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		DoClose(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	CSftpConnectOpData *pData = static_cast<CSftpConnectOpData *>(m_pCurOpData);
+	if (!pData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("m_pCurOpData of wrong type"));
+		DoClose(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	bool res;
+	switch (pData->opState)
+	{
+	case connect_keys:
+		res = Send(_T("keyfile \"") + pData->pKeyFiles->GetNextToken() + _T("\""));
+		break;
+	case connect_open:
+		res = Send(wxString::Format(_T("open \"%s@%s\" %d"), m_pCurrentServer->GetUser().c_str(), m_pCurrentServer->GetHost().c_str(), m_pCurrentServer->GetPort()));
+		break;
+	default:
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown op state: %d"), pData->opState);
+        DoClose(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
 
 	if (res)
 		return FZ_REPLY_WOULDBLOCK;
@@ -483,26 +563,24 @@ void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 						challenge += message->text;
 					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(challenge);
 					pNotification->server = *m_pCurrentServer;
-					pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
-					m_pCurOpData->waitForAsyncRequest = true;
-					m_pEngine->AddNotification(pNotification);
+
+					SendAsyncRequest(pNotification);
 				}
 				else
 				{
 					CSftpConnectOpData *pData = reinterpret_cast<CSftpConnectOpData*>(m_pCurOpData);
 
-					const wxString newChallenge = m_requestPreamble + _T("\n") + m_requestInstruction;
+					const wxString newChallenge = m_requestPreamble + _T("\n") + m_requestInstruction + message->text;
 
 					if (pData->pLastChallenge)
 					{
 						// Check for same challenge. Will most likely fail as well, so abort early.
 						if (*pData->pLastChallenge == newChallenge)
-						{
-							LogMessage(::Error, _T("Authentication failed."));
-							DoClose(FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED);
-							return;
-						}
-						delete pData->pLastChallenge;
+							LogMessage(::Error, _("Authentication failed."));
+						else
+							LogMessage(::Error, _("Server sent an additional login prompt. You need to use the interactive login type."));
+						DoClose(FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED);
+						return;
 					}
 
 					pData->pLastChallenge = new wxString(newChallenge);
@@ -526,6 +604,25 @@ void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 			{
 				long number = 0;
 				message->text.ToLong(&number);
+				
+				if (m_pTransferStatus && !m_pTransferStatus->madeProgress)
+				{
+					if (m_pCurOpData && m_pCurOpData->opId == cmd_transfer)
+					{
+						CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
+						if (pData->download)
+						{
+							if (number > 0)
+								SetTransferStatusMadeProgress();
+						}
+						else
+						{
+							if (m_pTransferStatus->currentOffset > m_pTransferStatus->startOffset + 65565)
+								SetTransferStatusMadeProgress();
+						}
+					}
+				}
+
 				UpdateTransferStatus(number);
 			}
 			break;
@@ -541,6 +638,27 @@ void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 		case sftpUsedQuotaSend:
 			OnQuotaRequest(CRateLimiter::outbound);
 			break;
+		case sftpKexAlgorithm:
+			m_sftpEncryptionDetails.kexAlgorithm = message->text;
+			break;
+		case sftpKexHash:
+			m_sftpEncryptionDetails.kexHash = message->text;
+			break;
+		case sftpCipherClientToServer:
+			m_sftpEncryptionDetails.cipherClientToServer = message->text;
+			break;
+		case sftpCipherServerToClient:
+			m_sftpEncryptionDetails.cipherServerToClient = message->text;
+			break;
+		case sftpMacClientToServer:
+			m_sftpEncryptionDetails.macClientToServer = message->text;
+			break;
+		case sftpMacServerToClient:
+			m_sftpEncryptionDetails.macServerToClient = message->text;
+			break;
+		case sftpHostkey:
+			m_sftpEncryptionDetails.hostKey = message->text;
+			break;
 		default:
 			wxFAIL_MSG(_T("given notification codes not handled"));
 			break;
@@ -623,16 +741,18 @@ bool CSftpControlSocket::AddToStream(const wxString& cmd, bool force_utf8 /*=fal
 	return true;
 }
 
-bool CSftpControlSocket::SendRequest(CAsyncRequestNotification *pNotification)
-{
-	pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
-	m_pEngine->AddNotification(pNotification);
-
-	return true;
-}
-
 bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
 {
+	if (m_pCurOpData)
+	{
+		if (!m_pCurOpData->waitForAsyncRequest)
+		{
+			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Not waiting for request reply, ignoring request reply %d"), pNotification->GetRequestID());
+			return false;
+		}
+		m_pCurOpData->waitForAsyncRequest = false;
+	}
+
 	const enum RequestId requestId = pNotification->GetRequestID();
 	switch(requestId)
 	{
@@ -646,13 +766,6 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 
 			CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 
-			if (!pData->waitForAsyncRequest)
-			{
-				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Not waiting for request reply, ignoring request reply %d"), pNotification->GetRequestID());
-				return false;
-			}
-			pData->waitForAsyncRequest = false;
-
 			CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
 			switch (pFileExistsNotification->overwriteAction)
 			{
@@ -822,6 +935,7 @@ public:
 		: COpData(cmd_list)
 	{
 		pParser = 0;
+		mtime_index = 0;
 	}
 
 	virtual ~CSftpListOpData()
@@ -837,16 +951,19 @@ public:
 	// Set to true to get a directory listing even if a cache
 	// lookup can be made after finding out true remote directory
 	bool refresh;
+
+	CDirectoryListing directoryListing;
+	int mtime_index;
 };
 
 enum listStates
 {
 	list_init = 0,
 	list_waitcwd,
-	list_list
+	list_list,
+	list_mtime
 };
 
-
 int CSftpControlSocket::List(CServerPath path /*=CServerPath()*/, wxString subDir /*=_T("")*/, bool refresh /*=false*/)
 {
 	LogMessage(Status, _("Retrieving directory listing..."));
@@ -886,12 +1003,6 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 {
 	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ListParseResponse(%s)"), reply.c_str());
 
-	if (!successful)
-	{
-		ResetOperation(FZ_REPLY_ERROR);
-		return FZ_REPLY_ERROR;
-	}
-
 	if (!m_pCurOpData)
 	{
 		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
@@ -907,28 +1018,101 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 		return FZ_REPLY_ERROR;
 	}
 
-	if (pData->opState != list_list)
+	if (pData->opState == list_list)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("ListParseResponse called at inproper time: %s"), pData->opState);
-		ResetOperation(FZ_REPLY_INTERNALERROR);
-		return FZ_REPLY_ERROR;
+		if (!successful)
+		{
+			ResetOperation(FZ_REPLY_ERROR);
+			return FZ_REPLY_ERROR;
+		}
+
+		if (!pData->pParser)
+		{
+			LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("pData->pParser is 0"));
+			ResetOperation(FZ_REPLY_INTERNALERROR);
+			return FZ_REPLY_ERROR;
+		}
+
+		pData->directoryListing = pData->pParser->Parse(m_CurrentPath);
+
+		int res = ListCheckTimezoneDetection();
+		if (res != FZ_REPLY_OK)
+			return res;
+
+		CDirectoryCache cache;
+		cache.Store(pData->directoryListing, *m_pCurrentServer, pData->path, pData->subDir);
+
+		m_pEngine->SendDirectoryListingNotification(m_CurrentPath, !pData->pNextOpData, true, false);
+
+		ResetOperation(FZ_REPLY_OK);
+		return FZ_REPLY_OK;
+	}
+	else if (pData->opState == list_mtime)
+	{
+		if (successful && reply != _T(""))
+		{
+			time_t seconds = 0;
+			bool parsed = true;
+			for (unsigned int i = 0; i < reply.Len(); i++)
+			{
+				wxChar c = reply[i];
+				if (c < '0' || c > '9')
+				{
+					parsed = false;
+					break;
+				}
+				seconds *= 10;
+				seconds += c - '0';
+			}
+			if (parsed)
+			{
+				wxDateTime date = wxDateTime(seconds);
+				if (date.IsValid())
+				{
+					date.MakeTimezone(wxDateTime::GMT0);
+					wxASSERT(pData->directoryListing[pData->mtime_index].hasTime);
+					wxDateTime listTime = pData->directoryListing[pData->mtime_index].time;
+					listTime -= wxTimeSpan(0, m_pCurrentServer->GetTimezoneOffset(), 0);
+
+					int serveroffset = (date - listTime).GetSeconds().GetLo();
+
+					wxDateTime now = wxDateTime::Now();
+					wxDateTime now_utc = now.ToTimezone(wxDateTime::GMT0);
+
+					int localoffset = (now - now_utc).GetSeconds().GetLo();
+					int offset = serveroffset + localoffset;
+
+					LogMessage(Status, _("Timezone offsets: Server: %d seconds. Local: %d seconds. Difference: %d seconds."), -serveroffset, localoffset, offset);
+
+					wxTimeSpan span(0, 0, offset);
+					const int count = pData->directoryListing.GetCount();
+					for (int i = 0; i < count; i++)
+					{
+						CDirentry& entry = pData->directoryListing[i];
+						if (!entry.hasTime)
+							continue;
+
+						entry.time += span;
+					}
+
+					// TODO: Correct cached listings
+
+					CServerCapabilities::SetCapability(*m_pCurrentServer, timezone_offset, yes, offset);
+				}
+			}
+		}
+
+		CDirectoryCache cache;
+		cache.Store(pData->directoryListing, *m_pCurrentServer, pData->path, pData->subDir);
+
+		m_pEngine->SendDirectoryListingNotification(m_CurrentPath, !pData->pNextOpData, true, false);
+
+		ResetOperation(FZ_REPLY_OK);
+		return FZ_REPLY_OK;
 	}
 
-	if (!pData->pParser)
-	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("pData->pParser is 0"));
-		return FZ_REPLY_INTERNALERROR;
-	}
-
-	CDirectoryListing listing = pData->pParser->Parse(m_CurrentPath);
-
-	CDirectoryCache cache;
-	cache.Store(listing, *m_pCurrentServer, pData->path, pData->subDir);
-
-	m_pEngine->SendDirectoryListingNotification(m_CurrentPath, !pData->pNextOpData, true, false);
-
-	ResetOperation(FZ_REPLY_OK);
-
+	LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("ListParseResponse called at inproper time: %d"), pData->opState);
+	ResetOperation(FZ_REPLY_INTERNALERROR);
 	return FZ_REPLY_ERROR;
 }
 
@@ -962,12 +1146,11 @@ int CSftpControlSocket::ListParseEntry(const wxString& entry)
 	if (pData->opState != list_list)
 	{
 		LogMessageRaw(RawList, entry);
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("ListParseResponse called at inproper time: %s"), pData->opState);
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("ListParseResponse called at inproper time: %d"), pData->opState);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-
 	if (!pData->pParser)
 	{
 		LogMessageRaw(RawList, entry);
@@ -1042,9 +1225,9 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 		}
 	}
 
-	if (!HasLock())
+	if (!pData->holdsLock)
 	{
-		if (!TryLockCache(m_CurrentPath))
+		if (!TryLockCache(lock_list, m_CurrentPath))
 			return FZ_REPLY_WOULDBLOCK;
 	}
 
@@ -1070,7 +1253,19 @@ int CSftpControlSocket::ListSend()
 	if (pData->opState == list_list)
 	{
 		pData->pParser = new CDirectoryListingParser(this, *m_pCurrentServer);
-		Send(_T("ls"));
+		pData->pParser->SetTimezoneOffset(GetTimezoneOffset());
+		if (!Send(_T("ls")))
+			return FZ_REPLY_ERROR;
+		return FZ_REPLY_WOULDBLOCK;
+	}
+	else if (pData->opState == list_mtime)
+	{
+		LogMessage(Status, _("Calculating timezone offset of server..."));
+		const wxString& name = pData->directoryListing[pData->mtime_index].name;
+		wxString quotedFilename = QuoteFilename(pData->directoryListing.path.FormatFilename(name, true));
+		if (!Send(_T("mtime ") + WildcardEscape(quotedFilename),
+			_T("mtime ") + quotedFilename))
+			return FZ_REPLY_ERROR;
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
@@ -1152,6 +1347,13 @@ int CSftpControlSocket::ChangeDir(CServerPath path /*=CServerPath()*/, wxString
 	pData->subDir = subDir;
 	pData->target = target;
 
+	if (pData->pNextOpData && pData->pNextOpData->opId == cmd_transfer &&
+		!static_cast<CSftpFileTransferOpData *>(pData->pNextOpData)->download)
+	{
+		pData->tryMkdOnFail = true;
+		wxASSERT(subDir == _T(""));
+	}
+
 	m_pCurOpData = pData;
 
 	return SendNextCommand();
@@ -1184,10 +1386,9 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 		if (!successful)
 		{
 			// Create remote directory if part of a file upload
-			if (pData->pNextOpData && pData->pNextOpData->opId == cmd_transfer &&
-				!static_cast<CSftpFileTransferOpData *>(pData->pNextOpData)->download && !pData->triedMkd)
+			if (pData->tryMkdOnFail)
 			{
-				pData->triedMkd = true;
+				pData->tryMkdOnFail = false;
 				int res = Mkdir(pData->path);
 				if (res != FZ_REPLY_OK)
 					return res;
@@ -1267,6 +1468,17 @@ int CSftpControlSocket::ChangeDirSend()
 		cmd = _T("pwd");
 		break;
 	case cwd_cwd:
+		if (pData->tryMkdOnFail && !pData->holdsLock)
+		{
+			if (IsLocked(lock_mkdir, pData->path))
+			{
+				// Some other engine is already creating this directory or
+				// performing an action that will lead to its creation
+				pData->tryMkdOnFail = false;
+			}
+			if (!TryLockCache(lock_mkdir, pData->path))
+				return FZ_REPLY_WOULDBLOCK;
+		}
 		cmd = _T("cd ") + QuoteFilename(pData->path.GetPath());
 		m_CurrentPath.Clear();
 		break;
@@ -1325,7 +1537,7 @@ int CSftpControlSocket::ResetOperation(int nErrorCode)
 	if (m_pCurOpData && m_pCurOpData->opId == cmd_connect)
 	{
 		CSftpConnectOpData *pData = static_cast<CSftpConnectOpData *>(m_pCurOpData);
-		if (!pData->gotInitialReply)
+		if (pData->opState == connect_init && (nErrorCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED)
 			LogMessage(::Error, _("fzsftp could not be started"));
 		if (pData->criticalFailure)
 			nErrorCode |= FZ_REPLY_CRITICALERROR;
@@ -1358,6 +1570,8 @@ int CSftpControlSocket::SendNextCommand()
 
 	switch (m_pCurOpData->opId)
 	{
+	case cmd_connect:
+		return ConnectSend();
 	case cmd_list:
 		return ListSend();
 	case cmd_transfer:
@@ -1455,78 +1669,119 @@ int CSftpControlSocket::FileTransferSubcommandResult(int prevResult)
 	{
 		if (prevResult == FZ_REPLY_OK)
 		{
-			pData->opState = filetransfer_waitlist;
-
 			CDirentry entry;
 			bool dirDidExist;
 			bool matchedCase;
 			CDirectoryCache cache;
 			bool found = cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dirDidExist, matchedCase);
-			bool shouldList = false;
 			if (!found)
 			{
 				if (!dirDidExist)
-					shouldList = true;
+					pData->opState = filetransfer_waitlist;
+				else if (pData->download &&
+					m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+				{
+					pData->opState = filetransfer_mtime;
+				}
+				else
+					pData->opState = filetransfer_transfer;
 			}
 			else
 			{
 				if (entry.unsure)
-					shouldList = true;
+					pData->opState = filetransfer_waitlist;
 				else
 				{
 					if (matchedCase)
+					{
 						pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
+						if (entry.hasDate)
+							pData->fileTime = entry.time;
+
+						if (pData->download &&
+							(!entry.hasDate || !entry.hasTime) &&
+							m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+						{
+							pData->opState = filetransfer_mtime;
+						}
+						else
+							pData->opState = filetransfer_transfer;
+					}
+					else
+						pData->opState = filetransfer_mtime;
 				}
 			}
-			if (shouldList)
+			if (pData->opState == filetransfer_waitlist)
 			{
 				int res = List(CServerPath(), _T(""), true);
 				if (res != FZ_REPLY_OK)
 					return res;
+				ResetOperation(FZ_REPLY_INTERNALERROR);
+				return FZ_REPLY_ERROR;
+			}
+			else if (pData->opState == filetransfer_transfer)
+			{
+				int res = CheckOverwriteFile();
+				if (res != FZ_REPLY_OK)
+					return res;
 			}
-
-			pData->opState = filetransfer_transfer;
-
-			int res = CheckOverwriteFile();
-			if (res != FZ_REPLY_OK)
-				return res;
 		}
 		else
 		{
 			pData->tryAbsolutePath = true;
-			pData->opState = filetransfer_transfer;
-
-			int res = CheckOverwriteFile();
-			if (res != FZ_REPLY_OK)
-				return res;
+			pData->opState = filetransfer_mtime;
 		}
 	}
 	else if (pData->opState == filetransfer_waitlist)
 	{
 		if (prevResult == FZ_REPLY_OK)
 		{
-			pData->opState = filetransfer_transfer;
-
 			CDirentry entry;
 			bool dirDidExist;
 			bool matchedCase;
 			CDirectoryCache cache;
 			bool found = cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dirDidExist, matchedCase);
-			if (found && matchedCase)
+			if (!found)
 			{
-				pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
+				if (!dirDidExist)
+					pData->opState = filetransfer_mtime;
+				else if (pData->download &&
+					m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+				{
+					pData->opState = filetransfer_mtime;
+				}
+				else
+					pData->opState = filetransfer_transfer;
+			}
+			else
+			{
+				if (matchedCase && !entry.unsure)
+				{
+					pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
+					if (entry.hasDate)
+						pData->fileTime = entry.time;
 
+					if (pData->download &&
+						(!entry.hasDate || !entry.hasTime) &&
+						m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+					{
+						pData->opState = filetransfer_mtime;
+					}
+					else
+						pData->opState = filetransfer_transfer;
+				}
+				else
+					pData->opState = filetransfer_mtime;
+			}
+			if (pData->opState == filetransfer_transfer)
+			{
 				int res = CheckOverwriteFile();
 				if (res != FZ_REPLY_OK)
 					return res;
 			}
 		}
-
-		pData->opState = filetransfer_transfer;
-
-		int res = CheckOverwriteFile();
-		if (res != FZ_REPLY_OK)
-			return res;
+		else
+			pData->opState = filetransfer_mtime;
 	}
 	else
 	{
@@ -1551,56 +1806,84 @@ int CSftpControlSocket::FileTransferSend()
 
 	CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 
-	wxString cmd;
-	if (pData->resume)
-		cmd = _T("re");
-	if (pData->download)
+	if (pData->opState == filetransfer_transfer)
 	{
-		// Create local directory
-		if (!pData->resume)
+		wxString cmd;
+		if (pData->resume)
+			cmd = _T("re");
+		if (pData->download)
 		{
-			wxFileName fn(pData->localFile);
-			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
+			// Create local directory
+			if (!pData->resume)
+			{
+				wxFileName fn(pData->localFile);
+				wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
+			}
+
+			InitTransferStatus(pData->remoteFileSize, pData->resume ? pData->localFileSize : 0, false);
+			cmd += _T("get ");
+			cmd += QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath)) + _T(" ");
+
+			wxString localFile = QuoteFilename(pData->localFile);
+			wxString logstr = cmd;
+			logstr += localFile;
+			LogMessageRaw(Command, logstr);
+
+			if (!AddToStream(cmd) || !AddToStream(localFile + _T("\n"), true))
+			{
+				ResetOperation(FZ_REPLY_ERROR);
+				return FZ_REPLY_ERROR;
+			}
 		}
-
-		InitTransferStatus(pData->remoteFileSize, pData->resume ? pData->localFileSize : 0, false);
-		cmd += _T("get ");
-		cmd += QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath)) + _T(" ");
-
-		wxString localFile = QuoteFilename(pData->localFile);
-		wxString logstr = cmd;
-		logstr += localFile;
-		LogMessageRaw(Command, logstr);
-
-		if (!AddToStream(cmd) || !AddToStream(localFile + _T("\n"), true))
+		else
 		{
-			ResetOperation(FZ_REPLY_ERROR);
+			InitTransferStatus(pData->localFileSize, pData->resume ? pData->remoteFileSize : 0, false);
+			cmd += _T("put ");
+
+			wxString logstr = cmd;
+			wxString localFile = QuoteFilename(pData->localFile) + _T(" ");
+			wxString remoteFile = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
+
+			logstr += localFile;
+			logstr += remoteFile;
+			LogMessageRaw(Command, logstr);
+
+			if (!AddToStream(cmd) || !AddToStream(localFile, true) ||
+				!AddToStream(remoteFile + _T("\n")))
+			{
+				ResetOperation(FZ_REPLY_ERROR);
+				return FZ_REPLY_ERROR;
+			}
+		}
+		SetTransferStatusStartTime();
+
+		pData->transferInitiated = true;
+	}
+	else if (pData->opState == filetransfer_mtime)
+	{
+		wxString quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
+		if (!Send(_T("mtime ") + WildcardEscape(quotedFilename),
+			_T("mtime ") + quotedFilename))
+			return FZ_REPLY_ERROR;
+	}
+	else if (pData->opState == filetransfer_chmtime)
+	{
+		wxASSERT(pData->fileTime.IsValid());
+		if (pData->download)
+		{
+			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("  filetransfer_chmtime during download"));
+			ResetOperation(FZ_REPLY_INTERNALERROR);
 			return FZ_REPLY_ERROR;
 		}
-	}
-	else
-	{
-		InitTransferStatus(pData->localFileSize, pData->resume ? pData->remoteFileSize : 0, false);
-		cmd += _T("put ");
 
-		wxString logstr = cmd;
-		wxString localFile = QuoteFilename(pData->localFile) + _T(" ");
-		wxString remoteFile = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
-
-		logstr += localFile;
-		logstr += remoteFile;
-		LogMessageRaw(Command, logstr);
-
-		if (!AddToStream(cmd) || !AddToStream(localFile, true) ||
-			!AddToStream(remoteFile + _T("\n")))
-		{
-			ResetOperation(FZ_REPLY_ERROR);
+		wxString quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
+		// Y2K38		
+		time_t ticks = pData->fileTime.GetTicks(); // Already in UTC
+		wxString seconds = wxString::Format(_T("%d"), (int)ticks);
+		if (!Send(_T("chmtime ") + seconds + _T(" ") + WildcardEscape(quotedFilename),
+			_T("chmtime ") + seconds + _T(" ") + quotedFilename))
 			return FZ_REPLY_ERROR;
-		}
 	}
-	SetTransferStatusStartTime();
-
-	pData->transferInitiated = true;
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -1609,12 +1892,6 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 {
 	LogMessage(Debug_Verbose, _T("FileTransferParseResponse()"));
 
-	if (!successful)
-	{
-		ResetOperation(FZ_REPLY_ERROR);
-		return FZ_REPLY_ERROR;
-	}
-
 	if (!m_pCurOpData)
 	{
 		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
@@ -1624,7 +1901,77 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 
 	CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 
-	if (pData->opState != filetransfer_transfer)
+	if (pData->opState == filetransfer_transfer)
+	{
+		if (!successful)
+		{
+			ResetOperation(FZ_REPLY_ERROR);
+			return FZ_REPLY_ERROR;
+		}
+
+		if (m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
+		{
+			wxFileName fn(pData->localFile);
+			if (fn.FileExists())
+			{
+				if (pData->download)
+				{
+					if (pData->fileTime.IsValid())
+						fn.SetTimes(&pData->fileTime, &pData->fileTime, 0);
+				}
+				else
+				{
+					pData->fileTime = fn.GetModificationTime();
+					if (pData->fileTime.IsValid())
+					{
+						pData->opState = filetransfer_chmtime;
+						return SendNextCommand();
+					}
+				}
+			}
+		}
+	}
+	else if (pData->opState == filetransfer_mtime)
+	{
+		if (successful && reply != _T(""))
+		{
+			time_t seconds = 0;
+			bool parsed = true;
+			for (unsigned int i = 0; i < reply.Len(); i++)
+			{
+				wxChar c = reply[i];
+				if (c < '0' || c > '9')
+				{
+					parsed = false;
+					break;
+				}
+				seconds *= 10;
+				seconds += c - '0';
+			}
+			if (parsed)
+			{
+				wxDateTime fileTime = wxDateTime(seconds);
+				if (fileTime.IsValid())
+					pData->fileTime = fileTime;
+			}
+		}
+		pData->opState = filetransfer_transfer;
+		int res = CheckOverwriteFile();
+		if (res != FZ_REPLY_OK)
+			return res;
+
+		return SendNextCommand();
+	}
+	else if (pData->opState == filetransfer_chmtime)
+	{
+		if (pData->download)
+		{
+			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("  filetransfer_chmtime during download"));
+			ResetOperation(FZ_REPLY_INTERNALERROR);
+			return FZ_REPLY_ERROR;
+		}
+	}
+	else
 	{
 		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("  Called at improper time: opState == %d"), pData->opState);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
@@ -1828,6 +2175,12 @@ int CSftpControlSocket::MkdirSend()
 	CMkdirOpData *pData = static_cast<CMkdirOpData *>(m_pCurOpData);
 	LogMessage(Debug_Debug, _T("  state = %d"), pData->opState);
 
+	if (!pData->holdsLock)
+	{
+		if (!TryLockCache(lock_mkdir, pData->path))
+			return FZ_REPLY_WOULDBLOCK;
+	}
+
 	bool res;
 	switch (pData->opState)
 	{
@@ -2337,14 +2690,19 @@ void CSftpControlSocket::OnRateAvailable(enum CRateLimiter::rate_direction direc
 
 void CSftpControlSocket::OnQuotaRequest(enum CRateLimiter::rate_direction direction)
 {
-	int bytes = GetAvailableBytes(direction);
+	wxLongLong bytes = GetAvailableBytes(direction);
 	if (bytes > 0)
 	{
-		AddToStream(wxString::Format(_T("-%d%d\n"), (int)direction, bytes));
-		UpdateUsage(direction, bytes);
+		int b;
+		if (bytes > INT_MAX)
+			b = INT_MAX;
+		else
+			b = bytes.GetLo();
+		AddToStream(wxString::Format(_T("-%d%d\n"), (int)direction, b));
+		UpdateUsage(direction, b);
 	}
 	else if (bytes < 0)
-		AddToStream(wxString::Format(_T("-%d-\n"), (int)direction, bytes));
+		AddToStream(wxString::Format(_T("-%d-\n"), (int)direction));
 	else
 		Wait(direction);
 }
@@ -2380,3 +2738,26 @@ int CSftpControlSocket::ParseSubcommandResult(int prevResult)
 
 	return FZ_REPLY_ERROR;
 }
+
+int CSftpControlSocket::ListCheckTimezoneDetection()
+{
+	wxASSERT(m_pCurOpData);
+
+	CSftpListOpData *pData = static_cast<CSftpListOpData *>(m_pCurOpData);
+
+	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset) == unknown)
+	{
+		const int count = pData->directoryListing.GetCount();
+		for (int i = 0; i < count; i++)
+		{
+			if (!pData->directoryListing[i].hasTime)
+				continue;
+
+			pData->opState = list_mtime;
+			pData->mtime_index = i;
+			return SendNextCommand();
+		}
+	}
+
+	return FZ_REPLY_OK;
+}
