@@ -22,7 +22,14 @@ typedef enum
 	sftpRequestPreamble,
 	sftpRequestInstruction,
 	sftpUsedQuotaRecv,
-	sftpUsedQuotaSend
+	sftpUsedQuotaSend,
+	sftpKexAlgorithm,
+	sftpKexHash,
+	sftpCipherClientToServer,
+	sftpCipherServerToClient,
+	sftpMacClientToServer,
+	sftpMacServerToClient,
+	sftpHostkey
 } sftpEventTypes;
 
 enum sftpRequestTypes
@@ -54,8 +61,6 @@ public:
 
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification);
 
-	bool SendRequest(CAsyncRequestNotification *pNotification);
-
 	void SetActive(bool recv);
 
 protected:
@@ -72,6 +77,7 @@ protected:
 	int ProcessReply(bool successful, const wxString& reply = _T(""));
 
 	int ConnectParseResponse(bool successful, const wxString& reply);
+	int ConnectSend();
 
 	virtual int FileTransfer(const wxString localFile, const CServerPath &remotePath,
 							 const wxString &remoteFile, bool download,
@@ -84,6 +90,7 @@ protected:
 	int ListSend();
 	int ListParseResponse(bool successful, const wxString& reply);
 	int ListParseEntry(const wxString& entry);
+	int ListCheckTimezoneDetection();
 
 	int ChangeDir(CServerPath path = CServerPath(), wxString subDir = _T(""));
 	int ChangeDirParseResponse(bool successful, const wxString& reply);
@@ -131,6 +138,8 @@ protected:
 
 	wxString m_requestPreamble;
 	wxString m_requestInstruction;
+
+	CSftpEncryptionNotification m_sftpEncryptionDetails;
 };
 
 #endif //__SFTPCONTROLSOCKET_H__
