@@ -45,7 +45,8 @@ enum NotificationId
 	nId_listing,		// directory listings
 	nId_asyncrequest,	// asynchronous request
 	nId_active,			// sent if data gets either received or sent
-	nId_data			// for memory downloads, indicates that new data is available.
+	nId_data,			// for memory downloads, indicates that new data is available.
+	nId_sftp_encryption	// information about key exchange, encryption algorithms and so on for SFTP
 };
 
 // Async request IDs
@@ -143,6 +144,8 @@ public:
 
 	bool ascii;
 
+	bool canResume;
+
 	// overwriteAction will be set by the request handler
 	enum OverwriteAction
 	{
@@ -150,9 +153,11 @@ public:
 		ask,
 		overwrite,
 		overwriteNewer,	// Overwrite if source file is newer than target file
-		resume,
+		resume, // Overwrites if cannot be resumed
 		rename,
-		skip
+		skip,
+
+		ACTION_COUNT
 	};
 
 	// Set overwriteAction to the desired action
@@ -202,6 +207,13 @@ public:
 	wxFileOffset totalSize;		// Total size of the file to transfer, -1 if unknown
 	wxFileOffset startOffset;
 	wxFileOffset currentOffset;
+
+	// True on download notifications iff currentOffset != startOffset.
+	// True on FTP upload notifications iff currentOffset != startOffset
+	// AND after the first accepted data after the first wxSOCKET_WOULDBLOCK.
+	// SFTP uploads: Set to true if currentOffset >= startOffset + 65536.
+	bool madeProgress;
+
 	bool list;
 };
 
@@ -263,6 +275,7 @@ protected:
 class CCertificateNotification : public CAsyncRequestNotification
 {
 public:
+	CCertificateNotification(const CCertificateNotification& ref);
 	CCertificateNotification(const wxString& host, unsigned int port,
 		const unsigned char* rawData, unsigned int len,
 		wxDateTime activationTime, wxDateTime expirationTime,
@@ -271,7 +284,9 @@ public:
 		const wxString& fingerprint_md5,
 		const wxString& fingerprint_sha1,
 		const wxString& subject,
-		const wxString& issuer);
+		const wxString& issuer,
+		const wxString& sessionCipher,
+		const wxString& sessionMac);
 	virtual ~CCertificateNotification();
 	virtual enum RequestId GetRequestID() const { return reqId_certificate; }
 
@@ -292,8 +307,13 @@ public:
 	const wxString& GetSubject() const { return m_subject; }
 	const wxString& GetIssuer() const { return m_issuer; }
 
+	const wxString& GetSessionCipher() const { return m_sessionCipher; }
+	const wxString& GetSessionMac() const { return m_sessionMac; }
+
 	bool m_trusted;
 
+	CCertificateNotification& operator=(const CCertificateNotification &op);
+
 protected:
 	wxString m_host;
 	unsigned int m_port;
@@ -313,6 +333,23 @@ protected:
 
 	wxString m_subject;
 	wxString m_issuer;
+
+	wxString m_sessionCipher;
+	wxString m_sessionMac;
+};
+
+class CSftpEncryptionNotification : public CNotification
+{
+public:
+	virtual enum NotificationId GetID() const { return nId_sftp_encryption; }
+
+	wxString hostKey;
+	wxString kexAlgorithm;
+	wxString kexHash;
+	wxString cipherClientToServer;
+	wxString cipherServerToClient;
+	wxString macClientToServer;
+	wxString macServerToClient;
 };
 
 #endif
