@@ -20,7 +20,11 @@ enum ServerType
 	MVS,
 	VXWORKS,
 	ZVM,
-	HPNONSTOP
+	HPNONSTOP,
+	DOS_VIRTUAL,
+	CYGWIN,
+
+	SERVERTYPE_MAX
 };
 
 enum LogonType
@@ -71,6 +75,7 @@ public:
 	int GetTimezoneOffset() const;
 	enum PasvMode GetPasvMode() const;
 	int MaximumMultipleConnections() const;
+	bool GetBypassProxy() const;
 	
 	// Return true if URL could be parsed correctly, false otherwise.
 	// If parsing fails, pError is filled with the reason and the CServer instance may be left an undefined state.
@@ -109,9 +114,21 @@ public:
 	static enum ServerProtocol GetProtocolFromPrefix(const wxString& prefix);
 	static wxString GetPrefixFromProtocol(const enum ServerProtocol protocol);
 
+	// Some protocol distinguish between ASCII and binary files for line-ending
+	// conversion.
+	static bool ProtocolHasDataTypeConcept(const enum ServerProtocol protocol);
+
 	// These commands will be executed after a successful login.
 	const std::vector<wxString>& GetPostLoginCommands() const { return m_postLoginCommands; }
 	bool SetPostLoginCommands(const std::vector<wxString>& postLoginCommands);
+	void SetBypassProxy(bool val);
+
+	// Abstract server name.
+	// Not compared in ==, < and related operators
+	void SetName(const wxString& name) { m_name = name; }
+	wxString GetName() const { return m_name; }
+
+	static wxString GetNameFromServerType(enum ServerType type);
 
 protected:
 	void Initialize();
@@ -129,8 +146,10 @@ protected:
 	int m_maximumMultipleConnections;
 	enum CharsetEncoding m_encodingType;
 	wxString m_customEncoding;
+	wxString m_name;
 
 	std::vector<wxString> m_postLoginCommands;
+	bool m_bypassProxy;
 };
 
 #endif
