@@ -9,6 +9,7 @@
 #endif
 #include "xmlfunctions.h"
 #include <wx/tokenzr.h>
+#include "cmdline.h"
 
 #include <wx/xrc/xh_bmpbt.h>
 #include <wx/xrc/xh_bttn.h>
@@ -71,12 +72,14 @@ CFileZillaApp::CFileZillaApp()
 {
 	m_pWrapEngine = 0;
 	m_pLocale = 0;
+	m_pCommandLine = 0;
 }
 
 CFileZillaApp::~CFileZillaApp()
 {
 	delete m_pLocale;
 	delete m_pWrapEngine;
+	delete m_pCommandLine;
 	COptions::Destroy();
 }
 
@@ -148,12 +151,22 @@ bool CFileZillaApp::OnInit()
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
 	//wxHandleFatalExceptions();
 #endif
-	wxApp::OnInit();
 
 	wxSystemOptions::SetOption(_T("msw.remap"), 0);
 	wxSystemOptions::SetOption(_T("mac.listctrl.always_use_generic"), 1);
 
+	int cmdline_result = ProcessCommandLine();
+	if (!cmdline_result)
+		return false;
+
 	LoadLocales();
+
+	if (cmdline_result < 0)
+	{
+		m_pCommandLine->DisplayUsage();
+		return false;
+	}
+
 	InitDefaultsDir();
 	InitSettingsDir();
 
@@ -580,6 +593,8 @@ bool CFileZillaApp::LoadLocales()
 {
 #ifndef __WXMAC__
 	m_localesDir = GetDataDir(_T("/../locale/*/filezilla.mo"));
+	if (m_localesDir == _T(""))
+		m_localesDir = GetDataDir(_T("/../locale/*/LC_MESSAGES/filezilla.mo"));
 	if (m_localesDir != _T(""))
 	{
 		if (m_localesDir[m_localesDir.Length() - 1] != wxFileName::GetPathSeparator())
@@ -722,9 +737,10 @@ void CFileZillaApp::CheckExistsFzsftp()
 	{
 		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable.c_str()),
 			_("File not found"), wxICON_ERROR);
+		executable.clear();
 	}
-#else
 
+#else
 
 	wxString program = _T("fzsftp");
 #ifdef __WXMSW__
@@ -815,8 +831,45 @@ void CFileZillaApp::CheckExistsFzsftp()
 	{
 		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program.c_str(), program.c_str(), program.c_str()),
 			_("File not found"), wxICON_ERROR);
+		executable.clear();
 	}
-	else
 #endif
-		COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
+
+	// Quote path if it contains spaces
+	if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
+		executable = _T("\"") + executable + _T("\"");
+
+	COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
+}
+
+#ifdef __WXMSW__
+extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM lParam)
+{
+	HWND child = FindWindowEx(hwnd, 0, 0, _T("FileZilla process identificator 3919DB0A-082D-4560-8E2F-381A35969FB4"));
+	if (child)
+	{
+		::PostMessage(hwnd, WM_ENDSESSION, (WPARAM)TRUE, (LPARAM)ENDSESSION_LOGOFF);
+	}
+
+	return TRUE;
+}
+#endif
+
+int CFileZillaApp::ProcessCommandLine()
+{
+	m_pCommandLine = new CCommandLine(argc, argv);
+	int res = m_pCommandLine->Parse() ? 1 : -1;
+
+	if (res > 0)
+	{
+		if (m_pCommandLine->HasSwitch(CCommandLine::close))
+		{
+#ifdef __WXMSW__
+			EnumWindows((WNDENUMPROC)EnumWindowCallback, 0);
+#endif
+			return 0;
+		}
+	}
+
+	return res;
 }
