@@ -18,6 +18,8 @@
 #endif
 #include "edithandler.h"
 #include "dragdropmanager.h"
+#include "local_filesys.h"
+#include "filelist_statusbar.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -41,8 +43,12 @@ public:
 		const int dropTarget = m_pLocalListView->m_dropTarget;
 		if (dropTarget != -1)
 		{
-			m_pLocalListView->SetItemState(dropTarget, 0, wxLIST_STATE_DROPHILITED);
 			m_pLocalListView->m_dropTarget = -1;
+#ifdef __WXMSW__
+			m_pLocalListView->SetItemState(dropTarget, 0, wxLIST_STATE_DROPHILITED);
+#else
+			m_pLocalListView->RefreshItem(dropTarget);
+#endif
 		}
 	}
 
@@ -151,8 +157,12 @@ public:
 			ClearDropHighlight();
 			if (hit != -1)
 			{
-				m_pLocalListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
 				m_pLocalListView->m_dropTarget = hit;
+#ifdef __WXMSW__
+				m_pLocalListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
+#else
+				m_pLocalListView->RefreshItem(hit);
+#endif
 			}
 		}
 
@@ -235,33 +245,23 @@ END_EVENT_TABLE()
 
 CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQueue)
 	: CFileListCtrl<CLocalFileData>(pParent, pState, pQueue),
-	CSystemImageList(16), CStateEventHandler(pState, STATECHANGE_LOCAL_DIR | STATECHANGE_APPLYFILTER | STATECHANGE_LOCAL_REFRESH_FILE)
+	CSystemImageList(16), CStateEventHandler(pState)
 {
+	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	m_pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
+
 	m_dropTarget = -1;
 
-	unsigned long widths[4] = { 120, 80, 100, 120 };
+	const unsigned long widths[4] = { 120, 80, 100, 120 };
 
-	if (!wxGetKeyState(WXK_SHIFT) || !wxGetKeyState(WXK_ALT) || !wxGetKeyState(WXK_CONTROL))
-		COptions::Get()->ReadColumnWidths(OPTION_LOCALFILELIST_COLUMN_WIDTHS, 4, widths);
+	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
+	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
+	AddColumn(_("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
+	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
+	LoadColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
 
-	InsertColumn(0, _("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
-	InsertColumn(1, _("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
-	InsertColumn(2, _("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
-	InsertColumn(3, _("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
-
-	wxString sortInfo = COptions::Get()->GetOption(OPTION_LOCALFILELIST_SORTORDER);
-	m_sortDirection = sortInfo[0] - '0';
-	if (m_sortDirection < 0 || m_sortDirection > 1)
-		m_sortDirection = 0;
-
-	if (sortInfo.Len() == 3)
-	{
-		m_sortColumn = sortInfo[2] - '0';
-		if (m_sortColumn < 0 || m_sortColumn > 3)
-			m_sortColumn = 0;
-	}
-	else
-		m_sortColumn = 0;
+	InitSort(OPTION_LOCALFILELIST_SORTORDER);
 
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
@@ -288,6 +288,8 @@ bool CLocalListView::DisplayDir(wxString dirname)
 	std::list<wxString> selectedNames;
 	if (m_dir != dirname)
 	{
+		ResetSearchPrefix();
+
 		if (IsComparing())
 			ExitComparisonMode();
 
@@ -295,11 +297,15 @@ bool CLocalListView::DisplayDir(wxString dirname)
 		int item = -1;
 		while (true)
 		{
-			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
+			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 			if (item == -1)
 				break;
-			SetItemState(item, 0, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
+			SetSelection(item, false);
+
 		}
+		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
+		if (item != -1)
+			SetItemState(item, 0, wxLIST_STATE_FOCUSED);
 		focused = _T("..");
 
 		if (GetItemCount())
@@ -312,6 +318,9 @@ bool CLocalListView::DisplayDir(wxString dirname)
 		selectedNames = RememberSelectedItems(focused);
 	}
 
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->UnselectAll();
+
 	const int oldItemCount = m_indexMapping.size();
 
 	m_fileData.clear();
@@ -352,69 +361,57 @@ bool CLocalListView::DisplayDir(wxString dirname)
 #ifdef __WXMSW__
 regular_dir:
 #endif
-		CFilterDialog filter;
+		CFilterManager filter;
+		CLocalFileSystem local_filesystem;
 
-		wxDir dir(dirname);
-		if (!dir.IsOpened())
+		if (!local_filesystem.BeginFindFiles(dirname, false))
 		{
 			SetItemCount(1);
 			return false;
 		}
-		wxString file;
-		bool found = dir.GetFirst(&file);
+
+		wxLongLong totalSize;
+		int unknown_sizes = 0;
+		int totalFileCount = 0;
+		int totalDirCount = 0;
+		int hidden = 0;
+
 		int num = m_fileData.size();
-		while (found)
+		CLocalFileData data;
+		data.flags = normal;
+		data.icon = -2;
+		bool wasLink;
+		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.lastModified, &data.attributes))
 		{
-			if (file == _T(""))
+			if (data.name == _T(""))
 			{
 				wxGetApp().DisplayEncodingWarning();
-				found = dir.GetNext(&file);
 				continue;
 			}
-			CLocalFileData data;
-
-			data.flags = normal;
-			data.dir = wxFileName::DirExists(dirname + file);
-			data.icon = -2;
-			data.name = file;
-
-			wxStructStat buf;
-			int result;
-			result = wxStat(dirname + file, &buf);
-
-			if (!result)
-			{
-				data.hasTime = true;
-				data.lastModified = wxDateTime(buf.st_mtime);
-			}
-			else
-				data.hasTime = false;
-
-			if (data.dir)
-				data.size = -1;
-			else
-				data.size = result ? -1 : buf.st_size;
-
-#ifdef __WXMSW__
-			DWORD tmp = GetFileAttributes(dirname + file);
-			if (tmp == INVALID_FILE_ATTRIBUTES)
-				data.attributes = 0;
-			else
-				data.attributes = tmp;
-#else
-			if (!result)
-				data.attributes = buf.st_mode & 0x777;
-			else
-				data.attributes = -1;
-#endif //__WXMSW__
+			data.hasTime = data.lastModified.IsValid();
 
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
+			{
+				if (data.dir)
+					totalDirCount++;
+				else
+				{
+					if (data.size != -1)
+						totalSize += data.size;
+					else
+						unknown_sizes++;
+					totalFileCount++;
+				}
 				m_indexMapping.push_back(num);
+			}
+			else
+				hidden++;
 			num++;
-
-			found = dir.GetNext(&file);
 		}
+
+		if (m_pFilelistStatusBar)
+			m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
 	}
 
 	if (m_dropTarget != -1)
@@ -446,7 +443,7 @@ regular_dir:
 	return true;
 }
 
-wxString FormatSize(const wxLongLong& size)
+wxString FormatSize(const wxLongLong& size, bool add_bytes_suffix = false)
 {
 	COptions* const pOptions = COptions::Get();
 	const int format = pOptions->GetOptionVal(OPTION_SIZE_FORMAT);
@@ -471,7 +468,15 @@ wxString FormatSize(const wxLongLong& size)
 		wxString tmp = size.ToString();
 		const int len = tmp.Len();
 		if (len <= 3)
-			return tmp;
+		{
+			if (!add_bytes_suffix)
+				return tmp;
+			else
+			{
+				const int last = (size % 1000000).GetLo();
+				return wxString::Format(wxPLURAL("%s byte", "%s bytes", last), tmp.c_str());
+			}
+		}
 
 		wxString result;
 		int i = (len - 1) % 3 + 1;
@@ -481,8 +486,13 @@ wxString FormatSize(const wxLongLong& size)
 			result += sep + tmp.Mid(i, 3);
 			i += 3;
 		}
-
-		return result;
+		if (!add_bytes_suffix)
+			return result;
+		else
+		{
+			const int last = (size % 1000000).GetLo();
+			return wxString::Format(wxPLURAL("%s byte", "%s bytes", last), result.c_str());
+		}
 	}
 
 	int divider;
@@ -491,10 +501,14 @@ wxString FormatSize(const wxLongLong& size)
 	else
 		divider = 1024;
 
+	// We always round up. Set to true if there's a reminder
 	bool r2 = false;
+
+	// Exponent (2^(10p) or 10^(3p) depending on option
 	int p = 0;
+
 	wxLongLong r = size;
-	while (r > divider || p == 6)
+	while (r > divider && p < 6)
 	{
 		const wxLongLong rr = r / divider;
 		if (rr * divider != r)
@@ -510,7 +524,7 @@ wxString FormatSize(const wxLongLong& size)
 	if (!p)
 		return result + _T("B");
 
-	// We stop at Exa. If someone has files bigger than that, he can afford to 
+	// We stop at Exa. If someone has files bigger than that, he can afford to
 	// make a donation to have this changed ;)
 	wxChar prefix[] = { ' ', 'K', 'M', 'G', 'T', 'P', 'E' };
 
@@ -653,6 +667,7 @@ void CLocalListView::DisplayDrives()
 	const wxChar* pDrive = drives;
 
 	int count = m_fileData.size();
+	int drive_count = 0;
 	while(*pDrive)
 	{
 		// Check if drive should be hidden by default
@@ -672,6 +687,8 @@ void CLocalListView::DisplayDrives()
 			}
 		}
 
+		drive_count++;
+
 		wxString path = pDrive;
 		if (path.Right(1) == _T("\\"))
 			path.Truncate(path.Length() - 1);
@@ -691,11 +708,19 @@ void CLocalListView::DisplayDrives()
 	}
 
 	delete [] drives;
+
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->SetDirectoryContents(0, drive_count, 0, false, 0);
 }
 
 void CLocalListView::DisplayShares(wxString computer)
 {
-	SHARE_INFO_1* pShareInfo = 0;
+	// Cast through a union to avoid warning about breaking strict aliasing rule
+	union
+	{
+		SHARE_INFO_1* pShareInfo;
+		LPBYTE pShareInfoBlob;
+	} si;
 
 	DWORD read, total;
 	DWORD resume_handle = 0;
@@ -704,16 +729,17 @@ void CLocalListView::DisplayShares(wxString computer)
 		computer.RemoveLast();
 
 	int j = m_fileData.size();
+	int share_count = 0;
 	int res = 0;
 	do
 	{
 		const wxWX2WCbuf buf = computer.wc_str(wxConvLocal);
-		res = NetShareEnum((wchar_t*)(const wchar_t*)buf, 1, (LPBYTE*)&pShareInfo, MAX_PREFERRED_LENGTH, &read, &total, &resume_handle);
+		res = NetShareEnum((wchar_t*)(const wchar_t*)buf, 1, &si.pShareInfoBlob, MAX_PREFERRED_LENGTH, &read, &total, &resume_handle);
 
 		if (res != ERROR_SUCCESS && res != ERROR_MORE_DATA)
 			break;
 
-		SHARE_INFO_1* p = pShareInfo;
+		SHARE_INFO_1* p = si.pShareInfo;
 		for (unsigned int i = 0; i < read; i++, p++)
 		{
 			if (p->shi1_type != STYPE_DISKTREE)
@@ -729,14 +755,19 @@ void CLocalListView::DisplayShares(wxString computer)
 
 			m_fileData.push_back(data);
 			m_indexMapping.push_back(j++);
+
+			share_count++;
 		}
 
-		NetApiBufferFree(pShareInfo);
+		NetApiBufferFree(si.pShareInfo);
 	}
 	while (res == ERROR_MORE_DATA);
+
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->SetDirectoryContents(0, share_count, 0, false, 0);
 }
 
-#endif
+#endif //__WXMSW__
 
 CLocalFileData* CLocalListView::GetData(unsigned int item)
 {
@@ -1025,10 +1056,24 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 
 void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 {
+	if (GetEditControl())
+	{
+		event.Skip();
+		return;
+	}
+
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALFILELIST"));
 	if (!pMenu)
 		return;
 
+	const bool connected = m_pState->IsRemoteConnected();
+	if (!connected)
+	{
+		pMenu->Enable(XRCID("ID_EDIT"), false);
+		pMenu->Enable(XRCID("ID_UPLOAD"), false);
+		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
+	}
+
 	int index = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	int count = 0;
 	int fillCount = 0;
@@ -1168,28 +1213,14 @@ void CLocalListView::OnMenuMkdir(wxCommandEvent& event)
 
 void CLocalListView::OnMenuDelete(wxCommandEvent& event)
 {
-	// Under Windows use SHFileOperation to delete files and directories.
-	// Under other systems, we have to recurse into subdirectories manually
-	// to delete all contents.
-
-#ifdef __WXMSW__
-	// SHFileOperation accepts a list of null-terminated strings. Go through all
-	// items to get the required buffer length
-
 	wxString dir = m_dir;
 	if (dir.Right(1) != _T("\\") && dir.Right(1) != _T("/"))
 		dir += _T("\\");
-	int dirLen = dir.Length();
-
-	int len = 1; // String list terminated by empty string
 
+	std::list<wxString> pathsToDelete;
 	long item = -1;
-	while (true)
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 	{
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
-			break;
-
 		if (!item && m_hasParent)
 			continue;
 
@@ -1200,130 +1231,11 @@ void CLocalListView::OnMenuDelete(wxCommandEvent& event)
 		if (data->flags == fill)
 			continue;
 
-		len += dirLen + data->name.Length() + 1;
+		pathsToDelete.push_back(dir + data->name);
 	}
+	if (!CLocalFileSystem::RecursiveDelete(pathsToDelete, this))
+		wxGetApp().DisplayEncodingWarning();
 
-	// Allocate memory
-	wxChar* pBuffer = new wxChar[len];
-	wxChar* p = pBuffer;
-
-	// Loop through all selected items again and fill buffer
-	item = -1;
-	while (true)
-	{
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
-			break;
-
-		if (!item && m_hasParent)
-			continue;
-
-		CLocalFileData *data = GetData(item);
-		if (!data)
-			continue;
-
-		if (data->flags == fill)
-			continue;
-
-		const wxString name = dir + data->name;
-		if (!wxFileName::FileExists(name) && !wxFileName::DirExists(name))
-			continue;
-
-		_tcscpy(p, dir);
-		p += dirLen;
-		_tcscpy(p, data->name);
-		p += data->name.Length() + 1;
-	}
-	if (p != pBuffer)
-	{
-		*p = 0;
-
-		// Now we can delete the files in the buffer
-		SHFILEOPSTRUCT op;
-		memset(&op, 0, sizeof(op));
-		op.hwnd = (HWND)GetHandle();
-		op.wFunc = FO_DELETE;
-		op.pFrom = pBuffer;
-
-		// Move to trash if shift is not pressed, else delete
-		op.fFlags = wxGetKeyState(WXK_SHIFT) ? 0 : FOF_ALLOWUNDO;
-
-		SHFileOperation(&op);
-	}
-	delete [] pBuffer;
-#else
-	if (wxMessageBox(_("Really delete all selected files and/or directories?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
-		return;
-
-	// Remember the directories to delete and the directories to visit
-	std::list<wxString> dirsToDelete;
-	std::list<wxString> dirsToVisit;
-
-	wxString dir = m_dir;
-	if (dir.Right(1) != _T("\\") && dir.Right(1) != _T("/"))
-		dir += _T("/");
-
-	// First process selected items
-	long item = -1;
-	while (true)
-	{
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
-			break;
-
-		if (!item && m_hasParent)
-			continue;
-
-		CLocalFileData *data = GetData(item);
-		if (!data)
-			continue;
-
-		if (data->flags == fill)
-			continue;
-
-		if (data->dir)
-		{
-			wxString subDir = dir + data->name + _T("/");
-			dirsToVisit.push_back(subDir);
-			dirsToDelete.push_front(subDir);
-		}
-		else
-			wxRemoveFile(dir + data->name);
-	}
-
-	// Process any subdirs which still have to be visited
-	while (!dirsToVisit.empty())
-	{
-		wxString filename = dirsToVisit.front();
-		dirsToVisit.pop_front();
-		wxDir dir;
-		if (!dir.Open(filename))
-			continue;
-
-		wxString file;
-		for (bool found = dir.GetFirst(&file); found; found = dir.GetNext(&file))
-		{
-			if (file == _T(""))
-			{
-				wxGetApp().DisplayEncodingWarning();
-				continue;
-			}
-			if (wxFileName::DirExists(filename + file))
-			{
-				wxString subDir = filename + file + _T("/");
-				dirsToVisit.push_back(subDir);
-				dirsToDelete.push_front(subDir);
-			}
-			else
-				wxRemoveFile(filename + file);
-		}
-	}
-
-	// Delete the now empty directories
-	for (std::list<wxString>::const_iterator iter = dirsToDelete.begin(); iter != dirsToDelete.end(); iter++)
-		wxRmdir(*iter);
-
-#endif
 	m_pState->SetLocalDir(m_dir);
 }
 
@@ -1354,7 +1266,7 @@ void CLocalListView::OnMenuRename(wxCommandEvent& event)
 
 void CLocalListView::OnChar(wxKeyEvent& event)
 {
-	int code = event.GetKeyCode();
+	const int code = event.GetKeyCode();
 	if (code == WXK_DELETE)
 	{
 		if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
@@ -1405,10 +1317,12 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 		{
 			CLocalFileData *data = GetData(i);
 			if (data && data->flags != fill)
-				SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+				SetSelection(i, true);
 			else
-				SetItemState(i, 0, wxLIST_STATE_SELECTED);
+				SetSelection(i, false);
 		}
+		if (m_pFilelistStatusBar)
+			m_pFilelistStatusBar->SelectAll();
 	}
 	else
 		OnChar(event);
@@ -1483,7 +1397,7 @@ bool Rename(wxWindow* parent, wxString dir, wxString from, wxString to)
 		return false;
 	}
 
-	return wxRename(dir + from, dir + to);
+	return wxRename(dir + from, dir + to) == 0;
 #endif
 }
 
@@ -1516,7 +1430,7 @@ void CLocalListView::OnEndLabelEdit(wxListEvent& event)
 	{
 		event.Veto();
 		return;
-	}	
+	}
 
 	wxString newname = event.GetLabel();
 #ifdef __WXMSW__
@@ -1537,6 +1451,11 @@ void CLocalListView::OnEndLabelEdit(wxListEvent& event)
 
 void CLocalListView::ApplyCurrentFilter()
 {
+	CFilterManager filter;
+
+	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing())
+		ExitComparisonMode();
+
 	unsigned int min = m_hasParent ? 1 : 0;
 	if (m_fileData.size() <= min)
 		return;
@@ -1544,18 +1463,47 @@ void CLocalListView::ApplyCurrentFilter()
 	wxString focused;
 	const std::list<wxString>& selectedNames = RememberSelectedItems(focused);
 
-	CFilterDialog filter;
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->UnselectAll();
+
+	wxLongLong totalSize;
+	int unknown_sizes = 0;
+	int totalFileCount = 0;
+	int totalDirCount = 0;
+	int hidden = 0;
+
 	m_indexMapping.clear();
 	if (m_hasParent)
 		m_indexMapping.push_back(0);
 	for (unsigned int i = min; i < m_fileData.size(); i++)
 	{
 		const CLocalFileData& data = m_fileData[i];
-		if (!filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
-			m_indexMapping.push_back(i);
+		if (data.flags == fill)
+			continue;
+		if (filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
+		{
+			hidden++;
+			continue;
+		}
+
+		if (data.dir)
+			totalDirCount++;
+		else
+		{
+			if (data.size != -1)
+				totalSize += data.size;
+			else
+				unknown_sizes++;
+			totalFileCount++;
+		}
+
+		m_indexMapping.push_back(i);
 	}
 	SetItemCount(m_indexMapping.size());
 
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+
 	SortList(-1, -1, false);
 
 	if (IsComparing())
@@ -1588,7 +1536,7 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 			else
 				selectedNames.push_back(_T("-") + data.name);
 		}
-		SetItemState(item, 0, wxLIST_STATE_SELECTED);
+		SetSelection(item, false);
 	}
 
 	item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
@@ -1642,14 +1590,18 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 			{
 				if (firstSelected == -1)
 					firstSelected = i;
-				SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+				if (m_pFilelistStatusBar)
+					m_pFilelistStatusBar->SelectDirectory();
+				SetSelection(i, true);
 				break;
 			}
 			else if (*iter == (_T("-") + data.name))
 			{
 				if (firstSelected == -1)
 					firstSelected = i;
-				SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+				if (m_pFilelistStatusBar)
+					m_pFilelistStatusBar->SelectFile(data.size);
+				SetSelection(i, true);
 				break;
 			}
 		}
@@ -1665,14 +1617,17 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	}
 }
 
-void CLocalListView::OnStateChange(unsigned int event, const wxString& data)
+void CLocalListView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
-	if (event == STATECHANGE_LOCAL_DIR)
+	if (notification == STATECHANGE_LOCAL_DIR)
 		DisplayDir(m_pState->GetLocalDir());
-	else if (event == STATECHANGE_APPLYFILTER)
+	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyCurrentFilter();
-	else if (event == STATECHANGE_LOCAL_REFRESH_FILE)
+	else
+	{
+		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
 		RefreshFile(data);
+	}
 }
 
 void CLocalListView::OnBeginDrag(wxListEvent& event)
@@ -1731,41 +1686,65 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 
 void CLocalListView::RefreshFile(const wxString& file)
 {
-	bool fileExists;
-	if (!(fileExists = wxFileName::FileExists(m_dir + file)) && !wxFileName::DirExists(m_dir + file))
-		return;
-
 	CLocalFileData data;
 
+	bool wasLink;
+	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir + file, wasLink, &data.size, &data.lastModified, &data.attributes);
+	if (type == CLocalFileSystem::unknown)
+		return;
+
 	data.flags = normal;
-	data.dir = !fileExists;
 	data.icon = -2;
 	data.name = file;
+	data.dir = type == CLocalFileSystem::dir;
+	data.hasTime = data.lastModified.IsValid();
 
-	wxStructStat buf;
-	int result = wxStat(m_dir + file, &buf);
-
-	if (!result)
-	{
-		data.hasTime = true;
-		data.lastModified = wxDateTime(buf.st_mtime);
-	}
-	else
-		data.hasTime = false;
-
-	if (data.dir)
-		data.size = -1;
-	else
-		data.size = result ? -1 : buf.st_size;
+	CFilterManager filter;
+	if (filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
+		return;
 
 	// Look if file data already exists
-	for (std::vector<CLocalFileData>::iterator iter = m_fileData.begin(); iter != m_fileData.end(); iter++)
+	unsigned int i = 0;
+	for (std::vector<CLocalFileData>::iterator iter = m_fileData.begin(); iter != m_fileData.end(); iter++, i++)
 	{
 		const CLocalFileData& oldData = *iter;
 		if (oldData.name != file)
 			continue;
 
-		// It exists, update entry
+		// Update file list status bar
+		if (m_pFilelistStatusBar)
+		{
+			int item = -1;
+			while (true)
+			{
+				item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+				if (item == -1)
+					break;
+				if (m_indexMapping[item] != i)
+					continue;
+
+				if (oldData.dir)
+					m_pFilelistStatusBar->UnselectDirectory();
+				else
+					m_pFilelistStatusBar->UnselectFile(oldData.size);
+				if (data.dir)
+					m_pFilelistStatusBar->SelectDirectory();
+				else
+					m_pFilelistStatusBar->SelectFile(data.size);
+				break;
+			}
+
+			if (oldData.dir)
+				m_pFilelistStatusBar->RemoveDirectory();
+			else
+				m_pFilelistStatusBar->RemoveFile(oldData.size);
+			if (data.dir)
+				m_pFilelistStatusBar->AddDirectory();
+			else
+				m_pFilelistStatusBar->AddFile(data.size);
+		}
+
+		// Update the data
 		data.fileType = oldData.fileType;
 
 		*iter = data;
@@ -1819,7 +1798,8 @@ void CLocalListView::RefreshFile(const wxString& file)
 			int state = GetItemState(i, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
 			if (state != prevState)
 			{
-				SetItemState(i, prevState, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
+				SetItemState(i, prevState, wxLIST_STATE_FOCUSED);
+				SetSelection(i, (prevState & wxLIST_STATE_SELECTED) != 0);
 				prevState = state;
 			}
 		}
@@ -1828,6 +1808,8 @@ void CLocalListView::RefreshFile(const wxString& file)
 	else
 	{
 		RefreshComparison();
+		if (m_pFilelistStatusBar)
+			m_pFilelistStatusBar->UnselectAll();
 		ReselectItems(selectedNames, focused);
 	}
 }
@@ -1836,7 +1818,7 @@ void CLocalListView::InitDateFormat()
 {
 	const wxString& dateFormat = COptions::Get()->GetOption(OPTION_DATE_FORMAT);
 	const wxString& timeFormat = COptions::Get()->GetOption(OPTION_TIME_FORMAT);
-	
+
 	if (dateFormat == _T("1"))
 		m_dateFormat = _T("%Y-%m-%d");
 	else if (dateFormat[0] == '2')
@@ -1861,7 +1843,12 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 
 	if (!data)
 		return 0;
-	
+
+#ifndef __WXMSW__
+	if (item == m_dropTarget)
+		return &pThis->m_dropHighlightAttribute;
+#endif
+
 	switch (data->flags)
 	{
 	case different:
@@ -2113,6 +2100,12 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 
 	wxString cmd = pEditHandler->GetSystemOpenCommand(fn.GetFullPath());
 	if (cmd == _T(""))
+	{
+		int pos = data->name.Find('.') == -1;
+		if (pos == -1 || (pos == 0 && data->name.Mid(1).Find('.') == -1))
+			cmd = pEditHandler->GetOpenCommand(fn.GetFullPath());
+	}
+	if (cmd == _T(""))
 	{
 		wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
 		return;
@@ -2123,3 +2116,13 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 
 	wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
 }
+
+bool CLocalListView::ItemIsDir(int index) const
+{
+	return m_fileData[index].dir;
+}
+
+wxLongLong CLocalListView::ItemGetSize(int index) const
+{
+	return m_fileData[index].size;
+}
