@@ -17,7 +17,7 @@ enum interfaceOptions
 	OPTION_UPDATECHECK_INTERVAL,
 	OPTION_UPDATECHECK_LASTDATE,
 	OPTION_UPDATECHECK_NEWVERSION,
-	OPTION_UPDATECHECK_URL,
+	OPTION_UPDATECHECK_CHECKBETA,
 	OPTION_DEBUG_MENU,
 	OPTION_FILEEXISTS_DOWNLOAD,
 	OPTION_FILEEXISTS_UPLOAD,
@@ -53,6 +53,12 @@ enum interfaceOptions
 	OPTION_SITEMANAGER_POSITION,
 	OPTION_THEME_ICONSIZE,
 	OPTION_MESSAGELOG_TIMESTAMP,
+	OPTION_SITEMANAGER_LASTSELECTED,
+	OPTION_LOCALFILELIST_COLUMN_SHOWN,
+	OPTION_REMOTEFILELIST_COLUMN_SHOWN,
+	OPTION_LOCALFILELIST_COLUMN_ORDER,
+	OPTION_REMOTEFILELIST_COLUMN_ORDER,
+	OPTION_FILELIST_STATUSBAR,
 
 	// Has to be last element
 	OPTIONS_NUM
@@ -87,10 +93,6 @@ public:
 	static void Destroy();
 
 	void Import(TiXmlElement* pElement);
-
-	// Columns in wxListCtrl
-	void SaveColumnWidths(const wxListCtrl* const pListCtrl, unsigned int optionId);
-	bool ReadColumnWidths(unsigned int optionId, unsigned int count, unsigned long* widths);
 	
 protected:
 	COptions();
