@@ -22,6 +22,8 @@
 #include <wx/utils.h>
 #include <wx/progdlg.h>
 #include <wx/sound.h>
+#include "local_filesys.h"
+#include "statusbar.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -164,6 +166,7 @@ EVT_MENU(XRCID("ID_ACTIONAFTER_SHUTDOWN"), CQueueView::OnActionAfter)
 EVT_COMMAND(wxID_ANY, fzEVT_ASKFORPASSWORD, CQueueView::OnAskPassword)
 
 EVT_TIMER(wxID_ANY, CQueueView::OnTimer)
+EVT_CHAR(CQueueView::OnChar)
 
 EVT_MENU(XRCID("ID_PRIORITY_HIGHEST"), CQueueView::OnSetPriority)
 EVT_MENU(XRCID("ID_PRIORITY_HIGH"), CQueueView::OnSetPriority)
@@ -315,32 +318,18 @@ protected:
 			{
 				const wxString& fullName = currentLocalPath + wxFileName::GetPathSeparator() + file;
 
-				if (wxDir::Exists(fullName))
+				bool isLink;
+				wxLongLong size;
+				int attributes;
+				CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(fullName, isLink, &size, 0, &attributes);
+
+				if (type == CLocalFileSystem::dir)
 				{
-					int attributes;
-#ifdef __WXMSW__
-					DWORD tmp = GetFileAttributes(fullName);
-					if (tmp == INVALID_FILE_ATTRIBUTES)
-						attributes = 0;
-					else
-						attributes = tmp;
-#else
-					wxStructStat buf;
-					const int result = wxLstat(fullName, &buf);
-					if (!result)
+					if (isLink)
 					{
-#ifdef S_ISLNK
-						if (S_ISLNK(buf.st_mode))
-						{
-							found = pDir->GetNext(&file);
-							continue;
-						}
-#endif //S_ISLNK
-						attributes = buf.st_mode & 0x777;
+						found = pDir->GetNext(&file);
+						continue;
 					}
-					else
-						attributes = -1;
-#endif //__WXMSWMM
 
 					if (!m_filters.FilenameFiltered(file, true, -1, true, attributes))
 					{
@@ -351,33 +340,15 @@ protected:
 						m_pFolderItem->m_dirsToCheck.push_back(pair);
 					}
 				}
-				else
+				else if (type == CLocalFileSystem::file)
 				{
-					wxStructStat buf;
-					int result;
-					result = wxStat(fullName, &buf);
-
-					int attributes;
-#ifdef __WXMSW__
-					DWORD tmp = GetFileAttributes(fullName);
-					if (tmp == INVALID_FILE_ATTRIBUTES)
-						attributes = 0;
-					else
-						attributes = tmp;
-#else
-					if (!result)
-						attributes = buf.st_mode & 0x777;
-					else
-						attributes = -1;
-#endif //__WXMSW__
-
-					if (!m_filters.FilenameFiltered(file, false, result ? -1 : buf.st_size, true, attributes))
+					if (!m_filters.FilenameFiltered(file, false, size, true, attributes))
 					{
 						t_newEntry entry;
 						entry.localFile = fullName.c_str();
 						entry.remoteFile = file.c_str();
 						entry.remotePath.SetSafePath(currentRemotePath.GetSafePath().c_str());
-						entry.size = result ? -1 : buf.st_size;
+						entry.size = size;
 
 						AddEntry(entry);
 					}
@@ -416,7 +387,7 @@ protected:
 	CQueueView* m_pOwner;
 	CFolderScanItem* m_pFolderItem;
 
-	CFilterDialog m_filters;
+	CFilterManager m_filters;
 
 	wxMutex m_sync;
 	wxCondition m_condition;
@@ -436,7 +407,7 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	m_activeCountDown = 0;
 	m_activeCountUp = 0;
 	m_activeMode = 0;
-	m_quit = false;
+	m_quit = 0;
 	m_waitStatusLineUpdate = false;
 	m_lastTopItem = -1;
 	m_pFolderProcessingThread = 0;
@@ -498,7 +469,7 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 	else
 	{
 		fileItem = new CFileItem(pServerItem, queueOnly, download, localFile, remoteFile, remotePath, size);
-		fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? remoteFile : wxFileName(localFile).GetFullName());
+		fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? remoteFile : wxFileName(localFile).GetFullName(), remotePath.GetType());
 		fileItem->m_edit = edit;
 	}
 
@@ -507,7 +478,10 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 	CommitChanges();
 
 	if (!m_activeMode && !queueOnly)
+	{
 		m_activeMode = 1;
+		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+	}
 
 	m_waitStatusLineUpdate = true;
 	AdvanceQueue();
@@ -532,7 +506,7 @@ bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, con
 
 		CFileItem* fileItem;
 		fileItem = new CFileItem(pServerItem, queueOnly, true, localPath + iter->name, iter->name, dataObject.GetServerPath(), iter->size);
-		fileItem->m_transferSettings.binary = ShouldUseBinaryMode(iter->name);
+		fileItem->m_transferSettings.binary = ShouldUseBinaryMode(iter->name, dataObject.GetServerPath().GetType());
 
 		InsertItem(pServerItem, fileItem);
 	}
@@ -540,7 +514,10 @@ bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, con
 	CommitChanges();
 
 	if (!m_activeMode && !queueOnly)
+	{
 		m_activeMode = 1;
+		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+	}
 
 	m_waitStatusLineUpdate = true;
 	AdvanceQueue();
@@ -617,9 +594,19 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 		{
 		case reqId_fileexists:
 			{
-				// Set default file exists action
 				CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
-				pFileExistsNotification->overwriteAction = (enum CFileExistsNotification::OverwriteAction)pEngineData->pItem->m_defaultFileExistsAction;
+				if (pFileExistsNotification->canResume &&
+					pEngineData->pItem->GetType() == QueueItemType_File &&
+					((CFileItem*)pEngineData->pItem)->m_autoResume &&
+					((CFileItem*)pEngineData->pItem)->m_transferSettings.binary)
+				{
+					pFileExistsNotification->overwriteAction = CFileExistsNotification::resume;
+				}
+				else
+				{
+					// Set default file exists action
+					pFileExistsNotification->overwriteAction = (enum CFileExistsNotification::OverwriteAction)pEngineData->pItem->m_defaultFileExistsAction;
+				}
 			}
 			break;
 		default:
@@ -645,7 +632,15 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 			const CTransferStatus *pStatus = pTransferStatusNotification->GetStatus();
 
 			if (pEngineData->active)
+			{
+				if (pStatus && pStatus->madeProgress && !pStatus->list && 
+					pEngineData->pItem->GetType() == QueueItemType_File)
+				{
+					CFileItem* pItem = (CFileItem*)pEngineData->pItem;
+					pItem->m_madeProgress = true;
+				}
 				pEngineData->pStatusLineCtrl->SetTransferStatus(pStatus);
+			}
 		}
 		delete pNotification;
 		break;
@@ -876,7 +871,15 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 		if (pEngineData->pItem && pEngineData->pItem->m_remove)
 			reason = remove;
 		else
+		{
+			if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->m_madeProgress)
+			{
+				CFileItem* pItem = (CFileItem*)pEngineData->pItem;
+				pItem->m_madeProgress = false;
+				pItem->m_autoResume = true;
+			}
 			reason = reset;
+		}
 		pEngineData->pItem->m_statusMessage = _("Interrupted by user");
 		ResetEngine(*pEngineData, reason);
 		return;
@@ -940,8 +943,14 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 		// Increase error count only if item didn't make any progress. This keeps
 		// user interaction at a minimum if connection is unstable.
 
-		// FIXME: Disabled since detection isn't reliable (yet)
-		//else if (!pEngineData->pStatusLineCtrl || !pEngineData->pStatusLineCtrl->MadeProgress())
+		if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->m_madeProgress)
+		{
+			// Don't increase error count if there has been progress
+			CFileItem* pItem = (CFileItem*)pEngineData->pItem;
+			pItem->m_madeProgress = false;
+			pItem->m_autoResume = true;
+		}
+		else
 		{
 			if ((replyCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
 				pEngineData->pItem->m_statusMessage = _T("");
@@ -1054,9 +1063,9 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 			UpdateSelections_ItemRemoved(GetItemIndex(data.pItem) + 1);
 
 			m_itemCount--;
-			SetItemCount(m_itemCount);
+			SaveSetItemCount(m_itemCount);
 
-			const CFileItem* const pFileItem = (CFileItem*)data.pItem;
+			CFileItem* const pFileItem = (CFileItem*)data.pItem;
 			if (pFileItem->Download())
 				m_pMainFrame->GetState()->RefreshLocalFile(pFileItem->GetLocalFile());
 
@@ -1069,6 +1078,14 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 					pEditHandler->FinishTransfer(reason == success, pFileItem->m_edit, fn.GetFullName());
 				else
 					pEditHandler->FinishTransfer(reason == success, pFileItem->m_edit, fn.GetFullPath());
+				if (reason == success)
+					pFileItem->m_edit = CEditHandler::none;
+			}
+
+			if (reason == failure)
+			{
+				pFileItem->m_autoResume = false;
+				pFileItem->m_madeProgress = false;
 			}
 		}
 
@@ -1390,12 +1407,15 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		UpdateStatusLinePositions();
 	}
 
+	m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+
 	return true;
 }
 
 bool CQueueView::Quit()
 {
-	m_quit = true;
+	if (!m_quit)
+		m_quit = 1;
 
 #ifdef __WXMSW__
 	if (m_actionAfterWarnDialog)
@@ -1428,9 +1448,13 @@ bool CQueueView::Quit()
 
 	DeleteEngines();
 
-	SaveQueue();
+	if (m_quit == 1)
+	{
+		SaveQueue();
+		m_quit = 2;
+	}
 
-	COptions::Get()->SaveColumnWidths(this, OPTION_QUEUE_COLUMN_WIDTHS);
+	SaveColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
 
 	return true;
 }
@@ -1463,6 +1487,9 @@ void CQueueView::CheckQueueState()
 				m_pQueue->SetSelection(2);
 		}
 		*/
+
+		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+
 		if (!m_quit)
 			ActionAfter();
 	}
@@ -1531,27 +1558,10 @@ void CQueueView::CalculateQueueSize()
 
 void CQueueView::DisplayQueueSize()
 {
-	wxStatusBar *pStatusBar = m_pMainFrame->GetStatusBar();
+	CStatusBar* pStatusBar = m_pMainFrame->GetStatusBar();
 	if (!pStatusBar)
 		return;
-
-	wxString queueSize;
-	wxLongLong totalSize = m_totalQueueSize;
-	if (totalSize == 0 && m_filesWithUnknownSize == 0)
-		queueSize = _("Queue: empty");
-	if (totalSize > (1000 * 1000))
-	{
-		totalSize /= 1000 * 1000;
-		queueSize.Printf(_("Queue: %s%d MB"), (m_filesWithUnknownSize > 0) ? _T(">") : _T(""), totalSize.GetLo());
-	}
-	else if (totalSize > 1000)
-	{
-		totalSize /= 1000;
-		queueSize.Printf(_("Queue: %s%d KB"), (m_filesWithUnknownSize > 0) ? _T(">") : _T(""), totalSize.GetLo());
-	}
-	else
-		queueSize.Printf(_("Queue: %s%d bytes"), (m_filesWithUnknownSize > 0) ? _T(">") : _T(""), totalSize.GetLo());
-	pStatusBar->SetStatusText(queueSize, 4);
+	pStatusBar->DisplayQueueSize(m_totalQueueSize, m_filesWithUnknownSize != 0);
 }
 
 bool CQueueView::QueueFolder(bool queueOnly, bool download, const wxString& localPath, const CServerPath& remotePath, const CServer& server)
@@ -1644,7 +1654,7 @@ bool CQueueView::QueueFiles(const std::list<t_newEntry> &entryList, bool queueOn
 		if (entry.localFile != _T(""))
 		{
 			fileItem = new CFileItem(pServerItem, queueOnly, download, entry.localFile, entry.remoteFile, entry.remotePath, entry.size);
-			fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? entry.remoteFile : wxFileName(entry.localFile).GetFullName());
+			fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? entry.remoteFile : wxFileName(entry.localFile).GetFullName(), entry.remotePath.GetType());
 			fileItem->m_defaultFileExistsAction = defaultFileExistsAction;
 		}
 		else
@@ -1656,7 +1666,10 @@ bool CQueueView::QueueFiles(const std::list<t_newEntry> &entryList, bool queueOn
 	CommitChanges();
 
 	if (!m_activeMode && !queueOnly)
+	{
 		m_activeMode = 1;
+		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+	}
 
 	m_waitStatusLineUpdate = true;
 	AdvanceQueue();
@@ -1883,7 +1896,10 @@ void CQueueView::SettingsChanged()
 	AdvanceQueue();
 }
 
-bool CQueueView::ShouldUseBinaryMode(wxString filename)
+// Defined in RemoteListView.cpp
+wxString StripVMSRevision(const wxString& name);
+
+bool CQueueView::ShouldUseBinaryMode(wxString filename, enum ServerType type)
 {
 	int mode = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
 	if (mode == 1)
@@ -1891,24 +1907,22 @@ bool CQueueView::ShouldUseBinaryMode(wxString filename)
 	else if (mode == 2)
 		return true;
 
-	int pos = filename.Find('.');
-	if (pos == -1)
-		return COptions::Get()->GetOptionVal(OPTION_ASCIINOEXT) == 0;
-	else if (!pos)
+	if (type == VMS)
+		filename = StripVMSRevision(filename);
+
+	if (filename[0] == '.')
 		return COptions::Get()->GetOptionVal(OPTION_ASCIIDOTFILE) == 0;
 
-	wxString ext = filename;
-	do
-	{
-		ext = ext.Mid(pos + 1);
-	}
-	while ((pos = ext.Find('.')) != -1);
+	int pos = filename.Find('.', true);
+	if (pos == -1)
+		return COptions::Get()->GetOptionVal(OPTION_ASCIINOEXT) == 0;
+	filename = filename.Mid(pos + 1);
 
-	if (ext == _T(""))
+	if (filename == _T(""))
 		return true;
 
 	for (std::list<wxString>::const_iterator iter = m_asciiFiles.begin(); iter != m_asciiFiles.end(); iter++)
-		if (!ext.CmpNoCase(*iter))
+		if (!filename.CmpNoCase(*iter))
 			return false;
 
 	return true;
@@ -2049,7 +2063,7 @@ void CQueueView::RemoveAll()
 			m_itemCount += 1 + (*iter)->GetChildrenCount(true);
 		}
 	}
-	SetItemCount(m_itemCount);
+	SaveSetItemCount(m_itemCount);
 	m_actionAfterState = ActionAfterState_Disabled;
 
 	m_serverList = newServerList;
@@ -2125,7 +2139,7 @@ void CQueueView::OnRemoveSelected(wxCommandEvent& event)
 	}
 	DisplayNumberQueuedFiles();
 	DisplayQueueSize();
-	SetItemCount(m_itemCount);
+	SaveSetItemCount(m_itemCount);
 
 	m_waitStatusLineUpdate = false;
 	UpdateStatusLinePositions();
@@ -2195,12 +2209,12 @@ bool CQueueView::StopItem(CServerItem* pServerItem)
 		if (RemoveItem(pItem, true, false))
 		{
 			DisplayNumberQueuedFiles();
-			SetItemCount(m_itemCount);
+			SaveSetItemCount(m_itemCount);
 			return true;
 		}
 	}
 	DisplayNumberQueuedFiles();
-	SetItemCount(m_itemCount);
+	SaveSetItemCount(m_itemCount);
 
 	return false;
 }
@@ -2791,7 +2805,9 @@ bool CQueueView::SwitchEngine(t_EngineData** ppEngineData)
 
 	t_EngineData* pEngineData = *ppEngineData;
 
-	for (std::vector<t_EngineData*>::iterator iter = ++m_engineData.begin(); iter != m_engineData.end(); iter++)
+	std::vector<t_EngineData*>::iterator iter = m_engineData.begin();
+	iter++;
+	for (; iter != m_engineData.end(); iter++)
 	{
 		t_EngineData* pNewEngineData = *iter;
 		if (pNewEngineData == pEngineData)
@@ -2863,3 +2879,14 @@ bool CQueueView::IsOtherEngineConnected(t_EngineData* pEngineData)
 
 	return false;
 }
+
+void CQueueView::OnChar(wxKeyEvent& event)
+{
+	if (event.GetKeyCode() == WXK_DELETE)
+	{
+		wxCommandEvent cmdEvt;
+		OnRemoveSelected(cmdEvt);
+	}
+	else
+		event.Skip();
+}
