@@ -75,7 +75,7 @@ public:
 
 	bool IsEmpty() const;
 	int IsActive() const { return m_activeMode; }
-	bool SetActive(const bool active = true);
+	bool SetActive(bool active = true);
 	bool Quit();
 
 	// If the settings are changed, this function will recalculate some
@@ -124,7 +124,7 @@ protected:
 		retry,
 		remove
 	};
-	
+
 	enum ActionAfterState GetActionAfterState() const;
 
 	void ResetEngine(t_EngineData& data, const enum ResetReason reason);
@@ -143,8 +143,8 @@ protected:
 	void CalculateQueueSize();
 	void DisplayQueueSize();
 	void SaveQueue();
-	bool ShouldUseBinaryMode(wxString filename);
-	
+	bool ShouldUseBinaryMode(wxString filename, enum ServerType type);
+
 	bool IsActionAfter(enum ActionAfterState);
 	void ActionAfter(bool warned = false);
 #ifdef __WXMSW__
@@ -188,7 +188,7 @@ protected:
 	int m_activeCountDown;
 	int m_activeCountUp;
 	int m_activeMode; // 0 inactive, 1 only immediate transfers, 2 all
-	bool m_quit;
+	int m_quit;
 
 	enum ActionAfterState m_actionAfterState;
 	wxString m_actionAfterRunCommand;
@@ -215,6 +215,7 @@ protected:
 	void OnEngineEvent(wxEvent &event);
 	void OnFolderThreadComplete(wxCommandEvent& event);
 	void OnFolderThreadFiles(wxCommandEvent& event);
+	void OnChar(wxKeyEvent& event);
 
 	// Context menu handlers
 	void OnContextMenu(wxContextMenuEvent& event);
