@@ -50,13 +50,16 @@ protected:
 	virtual wxListItemAttr* OnGetItemAttr(long item) const;
 	virtual int OnGetItemImage(long item) const;
 
+	virtual bool ItemIsDir(int index) const;
+	virtual wxLongLong ItemGetSize(int index) const;
+
 protected:
 	bool IsItemValid(unsigned int item) const;
 	int GetItemIndex(unsigned int item) const;
 
 	virtual CSortComparisonObject GetSortComparisonObject();
 
-	virtual void OnStateChange(unsigned int event, const wxString& data);
+	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
 	void ApplyCurrentFilter();
 	void SetDirectoryListing(const CDirectoryListing *pDirectoryListing, bool modified = false);
 	bool UpdateDirectoryListing(const CDirectoryListing *pDirectoryListing);
