@@ -227,8 +227,12 @@ END_EVENT_TABLE()
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue)
 	: wxTreeCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
 	CSystemImageList(16),
-	CStateEventHandler(pState, STATECHANGE_REMOTE_DIR | STATECHANGE_REMOTE_DIR_MODIFIED | STATECHANGE_APPLYFILTER)
+	CStateEventHandler(pState)
 {
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
+	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+
 	m_busy = false;
 	m_pQueue = pQueue;
 	AddRoot(_T(""));
@@ -246,11 +250,11 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(unsigned int event, const wxString& data)
+void CRemoteTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
-	if (event == STATECHANGE_REMOTE_DIR)
+	if (notification == STATECHANGE_REMOTE_DIR)
 		SetDirectoryListing(m_pState->GetRemoteDir(), false);
-	else if (event == STATECHANGE_REMOTE_DIR_MODIFIED)
+	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
 		SetDirectoryListing(m_pState->GetRemoteDir(), true);
 }
 
@@ -280,6 +284,13 @@ void CRemoteTreeView::SetDirectoryListing(const CDirectoryListing* pListing, boo
 	}
 	Enable(true);
 
+	if (pListing->m_hasUnsureEntries && !(pListing->m_hasUnsureEntries & ~(CDirectoryListing::unsure_unknown | CDirectoryListing::unsure_file_mask)))
+	{
+		// Just files changed, does not affect directory tree
+		m_busy = false;
+		return;
+	}
+
 	wxTreeItemId parent = MakeParent(pListing->path, !modified);
 	if (!parent)
 	{
@@ -290,7 +301,7 @@ void CRemoteTreeView::SetDirectoryListing(const CDirectoryListing* pListing, boo
 	if (!IsExpanded(parent) && parent != m_ExpandAfterList)
 	{
 		DeleteChildren(parent);
-		CFilterDialog filter;
+		CFilterManager filter;
 		if (HasSubdirs(*pListing, filter))
 			AppendItem(parent, _T(""), -1, -1);
 	}
@@ -434,7 +445,7 @@ void CRemoteTreeView::CreateImageList()
 	SetImageList(m_pImageList);
 }
 
-bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilterDialog& filter)
+bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter)
 {
 	if (!listing.m_hasDirs)
 		return false;
@@ -512,7 +523,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		return;
 	}
 
-	CFilterDialog filter;
+	CFilterManager filter;
 
 	std::list<wxString> dirs;
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
