@@ -6,6 +6,7 @@
 #endif
 
 class CWrapEngine;
+class CCommandLine;
 class CFileZillaApp : public wxApp
 {
 public:
@@ -29,12 +30,15 @@ public:
 
 	CWrapEngine* GetWrapEngine();
 
+	const CCommandLine* GetCommandLine() const { return m_pCommandLine; }
+
 protected:
 	bool InitDefaultsDir();
 	bool InitSettingsDir();
 	wxString GetSettingsDirFromDefaults();
 	bool LoadResourceFiles();
 	bool LoadLocales();
+	int ProcessCommandLine();
 
 	wxLocale* m_pLocale;
 
@@ -57,6 +61,8 @@ protected:
 	bool FileExists(const wxString& file) const;
 
 	CWrapEngine* m_pWrapEngine;
+
+	CCommandLine* m_pCommandLine;
 };
 
 DECLARE_APP(CFileZillaApp)
