@@ -280,7 +280,7 @@ void CFilterEditDialog::OnFilterTypeChange(wxCommandEvent& event)
 	MakeControls(filter, item);
 }
 
-void CFilterEditDialog::MakeControls(const CFilterCondition& condition, unsigned int i /*=-1*/)
+void CFilterEditDialog::MakeControls(const CFilterCondition& condition, int i /*=-1*/)
 {
 	wxRect size = m_pListCtrl->GetClientSize();
 
@@ -326,6 +326,9 @@ void CFilterEditDialog::MakeControls(const CFilterCondition& condition, unsigned
 	case permissions:
 		controls.pCondition->Create(m_pListCtrl, wxID_ANY, wxPoint(20 + typeRect.GetWidth(), posy), wxDefaultSize, permissionConditionTypes);
 		break;
+	default:
+		wxFAIL_MSG(_T("Unhandled condition"));
+		break;
 	}
 	controls.pCondition->Select(condition.condition);
 	wxRect conditionsRect = controls.pCondition->GetSize();
@@ -417,6 +420,9 @@ void CFilterEditDialog::SaveFilter(CFilter& filter)
 				condition.value = 1;
 			}
 			break;
+		default:
+			wxFAIL_MSG(_T("Unhandled condition"));
+			break;
 		}
 
 		condition.matchCase = filter.matchCase;
@@ -514,11 +520,15 @@ void CFilterEditDialog::OnDelete(wxCommandEvent& event)
 void CFilterEditDialog::OnRename(wxCommandEvent& event)
 {
 	const wxString& oldName = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
-	wxTextEntryDialog dlg(this, _("Please enter a new name for the filter."), _("Enter filter name"), oldName);
-	if (dlg.ShowModal() != wxID_OK)
+	wxTextEntryDialog *pDlg = new wxTextEntryDialog(this, _("Please enter a new name for the filter."), _("Enter filter name"), oldName);
+	if (pDlg->ShowModal() != wxID_OK)
+	{
+		delete pDlg;
 		return;
+	}
 
-	const wxString& newName = dlg.GetValue();
+	const wxString& newName = pDlg->GetValue();
+	delete pDlg;
 
 	if (newName == _T(""))
 	{
@@ -688,7 +698,7 @@ bool CFilterEditDialog::Validate()
 			wxMessageBox(_("At least one filter condition is incomplete"), _("Filter validation failed"), wxICON_ERROR, this);
 			return false;
 		}
-		else if (type == size)
+		else if (type == (enum t_filterType)::size)
 		{
 			long number;
 			if (!controls.pValue->GetValue().ToLong(&number) || number < 0)
