@@ -6,9 +6,6 @@
 		return false; \
 	XRCCTRL(*this, id, type)
 
-// Default title of the validation failed message box
-extern wxString validationFailed;
-
 class COptions;
 class CSettingsDialog;
 class COptionsPage : public wxPanel
@@ -32,6 +29,7 @@ public:
 	bool GetCheck(int id);
 	bool GetRCheck(int id);
 	wxString GetText(int id);
+	bool SetText(int id, const wxString& text, bool& failure);
 	wxString GetStaticText(int id);
 	int GetChoice(int id);
 
@@ -39,6 +37,10 @@ public:
 
 	void ReloadSettings();
 
+	// Always returns false
+	bool DisplayError(const wxString& controlToFocus, const wxString& error);
+	bool DisplayError(wxWindow* pWnd, const wxString& error);
+
 protected:
 	COptions* m_pOptions;
 	CSettingsDialog* m_pOwner;
