@@ -8,6 +8,7 @@ EVT_CONTEXT_MENU(CQueueViewFailed::OnContextMenu)
 EVT_MENU(XRCID("ID_REMOVEALL"), CQueueViewFailed::OnRemoveAll)
 EVT_MENU(XRCID("ID_REMOVE"), CQueueViewFailed::OnRemoveSelected)
 EVT_MENU(XRCID("ID_REQUEUE"), CQueueViewFailed::OnRequeueSelected)
+EVT_CHAR(CQueueViewFailed::OnChar)
 END_EVENT_TABLE()
 
 CQueueViewFailed::CQueueViewFailed(CQueue* parent, int index)
@@ -41,15 +42,19 @@ void CQueueViewFailed::OnRemoveAll(wxCommandEvent& event)
 	while ((item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 		SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
+	CEditHandler* pEditHandler = CEditHandler::Get();
+	if (pEditHandler)
+		pEditHandler->RemoveAll(CEditHandler::upload_and_remove_failed);
+
 	for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
 		delete *iter;
 	m_serverList.clear();
 
 	m_itemCount = 0;
-	SetItemCount(0);
+	SaveSetItemCount(0);
 	m_fileCount = 0;
 	m_folderScanCount = 0;
-	
+
 	DisplayNumberQueuedFiles();
 
 	Refresh();
@@ -72,12 +77,33 @@ void CQueueViewFailed::OnRemoveSelected(wxCommandEvent& event)
 		SetItemState(item, 0, wxLIST_STATE_SELECTED);
 	}
 
+	CEditHandler* pEditHandler = CEditHandler::Get();
+
 	while (!selectedItems.empty())
 	{
 		CQueueItem* pItem = selectedItems.front();
 		selectedItems.pop_front();
 
 		CQueueItem* pTopLevelItem = pItem->GetTopLevelItem();
+
+		if (pItem->GetType() == QueueItemType_Server)
+		{
+			CServerItem* pServerItem = (CServerItem*)pItem;
+			if (pEditHandler && pEditHandler->GetFileCount(CEditHandler::remote, CEditHandler::upload_and_remove_failed, &pServerItem->GetServer()))
+				pEditHandler->RemoveAll(CEditHandler::upload_and_remove_failed, &pServerItem->GetServer());
+		}
+		else if (pItem->GetType() == QueueItemType_File)
+		{
+			CFileItem* pFileItem = (CFileItem*)pItem;
+			if (pFileItem->m_edit == CEditHandler::remote && pEditHandler)
+			{
+				wxFileName fn(pFileItem->GetLocalFile());
+				enum CEditHandler::fileState state = pEditHandler->GetFileState(CEditHandler::remote, fn.GetFullName());
+				if (state == CEditHandler::upload_and_remove_failed)
+					pEditHandler->Remove(pFileItem->m_edit, fn.GetFullName());
+			}
+		}
+
 		if (!pTopLevelItem->GetChild(1))
 		{
 			// Parent will get deleted
@@ -88,7 +114,7 @@ void CQueueViewFailed::OnRemoveSelected(wxCommandEvent& event)
 		RemoveItem(pItem, true, false, false);
 	}
 	DisplayNumberQueuedFiles();
-	SetItemCount(m_itemCount);
+	SaveSetItemCount(m_itemCount);
 	Refresh();
 
 	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount())
@@ -129,7 +155,7 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 		if (pItem->GetType() == QueueItemType_Server)
 		{
 			CServerItem* pOldServerItem = (CServerItem*)pItem;
-			CServerItem* pServerItem = pQueueView->CreateServerItem(pOldServerItem->GetServer());			
+			CServerItem* pServerItem = pQueueView->CreateServerItem(pOldServerItem->GetServer());
 
 			unsigned int childrenCount = pOldServerItem->GetChildrenCount(false);
 			for (unsigned int i = 0; i < childrenCount; i++)
@@ -138,6 +164,13 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 				pFileItem->m_errorCount = 0;
 				pFileItem->m_statusMessage.Clear();
 
+				if (!pFileItem->Download() && !wxFileName::FileExists(pFileItem->GetLocalFile()))
+				{
+					failedToRequeueAll = true;
+					RemoveItem(pItem, true, false, false);
+					continue;
+				}
+
 				if (pFileItem->m_edit == CEditHandler::remote)
 				{
 					CEditHandler* pEditHandler = CEditHandler::Get();
@@ -148,8 +181,26 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 						continue;
 					}
 					wxFileName fn(pFileItem->GetLocalFile());
-					if (pEditHandler->GetFileState(CEditHandler::remote, fn.GetFullName()) != CEditHandler::unknown ||
-						!pEditHandler->AddFile(CEditHandler::remote, fn.GetFullName(), pFileItem->GetRemotePath(), pServerItem->GetServer()))
+					enum CEditHandler::fileState state = pEditHandler->GetFileState(CEditHandler::remote, fn.GetFullName());
+					if (state == CEditHandler::unknown)
+					{
+						wxASSERT(pFileItem->Download());
+						if (!pEditHandler->AddFile(CEditHandler::remote, fn.GetFullName(), pFileItem->GetRemotePath(), pServerItem->GetServer()))
+						{
+							failedToRequeueAll = true;
+							delete pFileItem;
+							continue;
+						}
+					}
+					else if (state == CEditHandler::upload_and_remove_failed)
+					{
+						wxASSERT(!pFileItem->Download());
+						if (!pEditHandler->UploadFile(pFileItem->m_edit, fn.GetFullName(), true))
+							failedToRequeueAll = true;
+						delete pFileItem;
+						continue;
+					}
+					else
 					{
 						failedToRequeueAll = true;
 						delete pFileItem;
@@ -160,7 +211,7 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 				pFileItem->SetParent(pServerItem);
 				pQueueView->InsertItem(pServerItem, pFileItem);
 			}
-			
+
 			m_fileCount -= childrenCount;
 			m_itemCount -= childrenCount + 1;
 			pOldServerItem->DetachChildren();
@@ -187,6 +238,13 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 			pFileItem->m_errorCount = 0;
 			pFileItem->m_statusMessage.Clear();
 
+			if (!pFileItem->Download() && !wxFileName::FileExists(pFileItem->GetLocalFile()))
+			{
+				failedToRequeueAll = true;
+				RemoveItem(pItem, true, false, false);
+				continue;
+			}
+
 			CServerItem* pOldServerItem = (CServerItem*)pItem->GetTopLevelItem();
 			CServerItem* pServerItem = pQueueView->CreateServerItem(pOldServerItem->GetServer());
 			RemoveItem(pItem, false, false, false);
@@ -201,14 +259,45 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 						pQueueView->CommitChanges();
 						pQueueView->RemoveItem(pServerItem, true, true, true);
 					}
-					
+
 					failedToRequeueAll = true;
 					delete pItem;
 					continue;
 				}
 				wxFileName fn(pFileItem->GetLocalFile());
-				if (pEditHandler->GetFileState(CEditHandler::remote, fn.GetFullName()) != CEditHandler::unknown ||
-					!pEditHandler->AddFile(CEditHandler::remote, fn.GetFullName(), pFileItem->GetRemotePath(), pServerItem->GetServer()))
+				enum CEditHandler::fileState state = pEditHandler->GetFileState(CEditHandler::remote, fn.GetFullName());
+				if (state == CEditHandler::unknown)
+				{
+					wxASSERT(pFileItem->Download());
+					if (!pEditHandler->AddFile(CEditHandler::remote, fn.GetFullName(), pFileItem->GetRemotePath(), pServerItem->GetServer()))
+					{
+						if (!pServerItem->GetChildrenCount(false))
+						{
+							pQueueView->CommitChanges();
+							pQueueView->RemoveItem(pServerItem, true, true, true);
+						}
+
+						failedToRequeueAll = true;
+						delete pItem;
+						continue;
+					}
+				}
+				else if (state == CEditHandler::upload_and_remove_failed)
+				{
+					wxASSERT(!pFileItem->Download());
+					if (!pEditHandler->UploadFile(pFileItem->m_edit, fn.GetFullName(), true))
+						failedToRequeueAll = true;
+
+					if (!pServerItem->GetChildrenCount(false))
+					{
+						pQueueView->CommitChanges();
+						pQueueView->RemoveItem(pServerItem, true, true, true);
+					}
+
+					delete pItem;
+					continue;
+				}
+				else
 				{
 					if (!pServerItem->GetChildrenCount(false))
 					{
@@ -224,14 +313,14 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 
 			pItem->SetParent(pServerItem);
 			pQueueView->InsertItem(pServerItem, pItem);
-		}		
+		}
 	}
 	m_fileCountChanged = true;
 
 	pQueueView->CommitChanges();
 
 	DisplayNumberQueuedFiles();
-	SetItemCount(m_itemCount);
+	SaveSetItemCount(m_itemCount);
 	Refresh();
 
 	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount())
@@ -240,3 +329,14 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 	if (failedToRequeueAll)
 		wxMessageBox(_("Not all items could be requeued for viewing / editing."));
 }
+
+void CQueueViewFailed::OnChar(wxKeyEvent& event)
+{
+	if (event.GetKeyCode() == WXK_DELETE)
+	{
+		wxCommandEvent cmdEvt;
+		OnRemoveSelected(cmdEvt);
+	}
+	else
+		event.Skip();
+}
