@@ -45,9 +45,10 @@ public:
 	{
 	}
 
-	virtual size_t GetDataSize() const { return 0; }
+	// GTK doesn't like data size of 0
+	virtual size_t GetDataSize() const { return 1; }
 
-	virtual bool GetDataHere(void *buf) const { return true; }
+	virtual bool GetDataHere(void *buf) const { memset(buf, 0, 1); return true; }
 
 	virtual bool SetData(size_t len, const void *buf) { return true; }
 };
@@ -105,29 +106,23 @@ public:
 		return def;
 	}
 
-	virtual wxDragResult OnDrop(wxCoord x, wxCoord y, wxDragResult def)
+	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
 		ClearDropHighlight();
-		if (def == wxDragError ||
-			def == wxDragNone ||
-			def == wxDragCancel)
-		{
-			return def;
-		}
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
 		if (!hit)
-			return wxDragNone;
+			return false;
 		if (hit == m_pSiteManager->m_dropSource)
-			return wxDragNone;
+			return false;
 
 		const bool predefined = m_pSiteManager->IsPredefinedItem(hit);
 		if (predefined)
-			return wxDragNone;
+			return false;
 
 		wxTreeCtrl *pTree = XRCCTRL(*m_pSiteManager, "ID_SITETREE", wxTreeCtrl);
 		if (pTree->GetItemData(hit))
-			return wxDragNone;
+			return false;
 
 		wxTreeItemId item = hit;
 		while (item != pTree->GetRootItem())
@@ -135,15 +130,12 @@ public:
 			if (item == m_pSiteManager->m_dropSource)
 			{
 				ClearDropHighlight();
-				return wxDragNone;
+				return false;
 			}
 			item = pTree->GetItemParent(item);
 		}
 
-		if (def == wxDragMove && pTree->GetItemParent(m_pSiteManager->m_dropSource) == hit)
-			return wxDragNone;
-
-		return def;
+		return true;
 	}
 
 	virtual void OnLeave()
@@ -269,7 +261,7 @@ CSiteManager::~CSiteManager()
 	}
 }
 
-bool CSiteManager::Create(wxWindow* parent)
+bool CSiteManager::Create(wxWindow* parent, const CServer* pServer /*=0*/)
 {
 	m_pSiteManagerMutex = new CInterProcessMutex(MUTEX_SITEMANAGERGLOBAL, false);
 	if (!m_pSiteManagerMutex->TryLock())
@@ -328,6 +320,9 @@ bool CSiteManager::Create(wxWindow* parent)
 		XRCCTRL(*this, "wxID_OK", wxButton)->SetFocus();
 #endif
 
+	if (pServer)
+		CopyAddServer(*pServer);
+
 	return true;
 }
 
@@ -343,11 +338,8 @@ void CSiteManager::CreateControls(wxWindow* parent)
 
 	wxChoice *pChoice = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice);
 	wxASSERT(pChoice);
-	pChoice->Append(_T("Unix"));
-	pChoice->Append(_T("VMS"));
-	pChoice->Append(_T("MVS"));
-	pChoice->Append(_T("Dos"));
-	pChoice->Append(_T("VxWorks"));
+	for (int i = 0; i < SERVERTYPE_MAX; i++)
+		pChoice->Append(CServer::GetNameFromServerType((enum ServerType)i));
 }
 
 void CSiteManager::OnOK(wxCommandEvent& event)
@@ -359,6 +351,8 @@ void CSiteManager::OnOK(wxCommandEvent& event)
 
 	Save();
 
+	RememberLastSelected();
+
 	EndModal(wxID_OK);
 }
 
@@ -394,6 +388,8 @@ void CSiteManager::OnConnect(wxCommandEvent& event)
 
 	Save();
 
+	RememberLastSelected();
+
 	EndModal(wxID_YES);
 }
 
@@ -413,9 +409,12 @@ public:
 class CSiteManagerXmlHandler_Tree : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Tree(wxTreeCtrl* pTree, wxTreeItemId root)
+	CSiteManagerXmlHandler_Tree(wxTreeCtrl* pTree, wxTreeItemId root, const wxString& lastSelection)
 		: m_pTree(pTree), m_item(root)
 	{
+		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection))
+			m_lastSelection.clear();
+		m_wrong_sel_depth = 0;
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -433,18 +432,46 @@ public:
 		m_item = newItem;
 		m_expand.push_back(expanded);
 
+		if (!m_wrong_sel_depth && !m_lastSelection.empty())
+		{
+			const wxString& first = m_lastSelection.front();
+			if (first == name)
+			{
+				m_lastSelection.pop_front();
+				if (m_lastSelection.empty())
+					m_pTree->SelectItem(newItem);
+			}
+			else
+				m_wrong_sel_depth++;
+		}
+		else
+			m_wrong_sel_depth++;
+
 		return true;
 	}
 
 	virtual bool AddSite(const wxString& name, CSiteManagerItemData* data)
 	{
-		m_pTree->AppendItem(m_item, name, 2, 2, data);
+		wxTreeItemId newItem = m_pTree->AppendItem(m_item, name, 2, 2, data);
+
+		if (!m_wrong_sel_depth && !m_lastSelection.empty())
+		{
+			const wxString& first = m_lastSelection.front();
+			if (first == name)
+			{
+				m_lastSelection.clear();
+				m_pTree->SelectItem(newItem);
+			}
+		}
 
 		return true;
 	}
 
 	virtual bool LevelUp()
 	{
+		if (m_wrong_sel_depth)
+			m_wrong_sel_depth--;
+
 		if (!m_expand.empty())
 		{
 			const bool expand = m_expand.back();
@@ -466,6 +493,9 @@ protected:
 	wxTreeCtrl* m_pTree;
 	wxTreeItemId m_item;
 
+	std::list<wxString> m_lastSelection;
+	int m_wrong_sel_depth;
+
 	std::list<bool> m_expand;
 };
 
@@ -489,7 +519,6 @@ bool CSiteManager::Load()
 		m_ownSites = pTree->AddRoot(_("My Sites"), 0, 0);
 
 	wxTreeItemId treeId = m_ownSites;
-	pTree->SelectItem(treeId);
 	pTree->SetItemImage(treeId, 1, wxTreeItemIcon_Expanded);
 	pTree->SetItemImage(treeId, 1, wxTreeItemIcon_SelectedExpanded);
 
@@ -507,13 +536,26 @@ bool CSiteManager::Load()
 	if (!pElement)
 		return true;
 
-	CSiteManagerXmlHandler_Tree handler(pTree, treeId);
+	wxString lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
+	if (lastSelection[0] == '0')
+	{
+		if (lastSelection == _T("0"))
+			pTree->SelectItem(treeId);
+		else
+			lastSelection = lastSelection.Mid(1);
+	}
+	else
+		lastSelection = _T("");
+	CSiteManagerXmlHandler_Tree handler(pTree, treeId, lastSelection);
 
 	bool res = Load(pElement, &handler);
 
 	pTree->SortChildren(treeId);
 	pTree->Expand(treeId);
-	pTree->SelectItem(treeId);
+	if (!pTree->GetSelection())
+		pTree->SelectItem(treeId);
+
+	pTree->EnsureVisible(pTree->GetSelection());
 
 	return res;
 }
@@ -744,6 +786,17 @@ void CSiteManager::OnNewFolder(wxCommandEvent&event)
 	pTree->EditLabel(newItem);
 }
 
+static enum ServerType GetTypeFromName(const wxString& name)
+{
+	for (int i = 0; i < SERVERTYPE_MAX; i++)
+	{
+		enum ServerType type = (enum ServerType)i;
+		if (name == CServer::GetNameFromServerType(type))
+			return type;
+	}
+
+	return DEFAULT;
+}
 
 bool CSiteManager::Verify()
 {
@@ -828,6 +881,21 @@ bool CSiteManager::Verify()
 		return false;
 	}
 
+	const wxString remotePathRaw = XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->GetValue();
+	if (remotePathRaw != _T(""))
+	{
+		CServerPath remotePath;
+		const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
+		remotePath.SetType(GetTypeFromName(serverType));
+		if (!remotePath.SetPath(remotePathRaw))
+		{
+			XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetFocus();
+			wxMessageBox(_("Default remote path cannot be parsed. Make sure it is valid and is supported by the selected servertype."));
+			return false;
+		}
+	}
+
+
 	return true;
 }
 
@@ -937,6 +1005,10 @@ void CSiteManager::OnDelete(wxCommandEvent& event)
 	if (!dlg.Run())
 		return;
 
+	wxTreeItemId parent = pTree->GetItemParent(item);
+	if (pTree->GetChildrenCount(parent) == 1)
+		pTree->Collapse(parent);
+
 	pTree->Delete(item);
 }
 
@@ -976,37 +1048,8 @@ void CSiteManager::OnNewSite(wxCommandEvent& event)
 	if (!Verify())
 		return;
 
-	wxString newName = _("New site");
-	int index = 2;
-	while (true)
-	{
-		wxTreeItemId child;
-		wxTreeItemIdValue cookie;
-		child = pTree->GetFirstChild(item, cookie);
-		bool found = false;
-		while (child.IsOk())
-		{
-			wxString name = pTree->GetItemText(child);
-			int cmp = name.CmpNoCase(newName);
-			if (!cmp)
-			{
-				found = true;
-				break;
-			}
-
-			child = pTree->GetNextChild(item, cookie);
-		}
-		if (!found)
-			break;
-
-		newName = _("New site") + wxString::Format(_T(" %d"), index++);
-	}
-
-	wxTreeItemId newItem = pTree->AppendItem(item, newName, 2, 2, new CSiteManagerItemData());
-	pTree->SortChildren(item);
-	pTree->SelectItem(newItem);
-	pTree->Expand(item);
-	pTree->EditLabel(newItem);
+	CServer server;
+	AddNewSite(item, server);
 }
 
 void CSiteManager::OnLogontypeSelChanged(wxCommandEvent& event)
@@ -1056,7 +1099,7 @@ bool CSiteManager::UpdateServer()
 	else
 		data->m_server.SetProtocol(FTP);
 
-	wxString logonType = XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection();
+	const wxString logonType = XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection();
 	if (logonType == _("Normal"))
 		data->m_server.SetLogonType(NORMAL);
 	else if (logonType == _("Ask for password"))
@@ -1074,19 +1117,8 @@ bool CSiteManager::UpdateServer()
 
 	data->m_comments = XRCCTRL(*this, "ID_COMMENTS", wxTextCtrl)->GetValue();
 
-	wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
-	if (serverType == _T("Unix"))
-		data->m_server.SetType(UNIX);
-	else if (serverType == _T("Dos"))
-		data->m_server.SetType(DOS);
-	else if (serverType == _T("VMS"))
-		data->m_server.SetType(VMS);
-	else if (serverType == _T("MVS"))
-		data->m_server.SetType(MVS);
-	else if (serverType == _T("VxWorks"))
-		data->m_server.SetType(VXWORKS);
-	else
-		data->m_server.SetType(DEFAULT);
+	const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
+	data->m_server.SetType(GetTypeFromName(serverType));
 
 	data->m_localDir = XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->GetValue();
 	data->m_remoteDir = CServerPath();
@@ -1122,6 +1154,13 @@ bool CSiteManager::UpdateServer()
 	else
 		data->m_server.SetEncodingType(ENCODING_AUTO);
 
+	if (XRCCTRL(*this, "ID_BYPASSPROXY", wxCheckBox)->GetValue())
+		data->m_server.SetBypassProxy(true);
+	else
+		data->m_server.SetBypassProxy(false);
+
+	data->m_server.SetName(pTree->GetItemText(item));
+
 	return true;
 }
 
@@ -1140,7 +1179,6 @@ bool CSiteManager::GetServer(CSiteManagerItemData& data)
 		return false;
 
 	data = *pData;
-	data.m_name = pTree->GetItemText(item);
 
 	return true;
 }
@@ -1223,13 +1261,14 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_PORT", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->SetStringSelection(_("FTP"));
+		XRCCTRL(*this, "ID_BYPASSPROXY", wxCheckBox)->SetValue(false);
 		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(_("Anonymous"));
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_COMMENTS", wxTextCtrl)->SetValue(_T(""));
 
-		XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_("Default"));
+		XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetSelection(0);
 		XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetValue(_T(""));
 		XRCCTRL(*this, "ID_TIMEZONE_HOURS", wxSpinCtrl)->SetValue(0);
@@ -1272,6 +1311,7 @@ void CSiteManager::SetCtrlState()
 		else
 			XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->SetStringSelection(CServer::GetProtocolName(FTP));
 		XRCCTRL(*this, "ID_PROTOCOL", wxWindow)->Enable(!predefined);
+		XRCCTRL(*this, "ID_BYPASSPROXY", wxCheckBox)->SetValue(data->m_server.GetBypassProxy());
 
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->Enable(!predefined && data->m_server.GetLogonType() != ANONYMOUS);
 		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->Enable(!predefined && data->m_server.GetLogonType() == NORMAL || data->m_server.GetLogonType() == ACCOUNT);
@@ -1303,27 +1343,7 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_COMMENTS", wxTextCtrl)->SetValue(data->m_comments);
 		XRCCTRL(*this, "ID_COMMENTS", wxWindow)->Enable(!predefined);
 
-		switch (data->m_server.GetType())
-		{
-		case UNIX:
-			XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_T("Unix"));
-			break;
-		case DOS:
-			XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_T("Dos"));
-			break;
-		case MVS:
-			XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_T("MVS"));
-			break;
-		case VMS:
-			XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_T("VMS"));
-			break;
-		case VXWORKS:
-			XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_T("VxWorks"));
-			break;
-		default:
-			XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetStringSelection(_("Default"));
-			break;
-		}
+		XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->SetSelection(data->m_server.GetType());
 		XRCCTRL(*this, "ID_SERVERTYPE", wxWindow)->Enable(!predefined);
 		XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->SetValue(data->m_localDir);
 		XRCCTRL(*this, "ID_LOCALDIR", wxWindow)->Enable(!predefined);
@@ -1497,7 +1517,17 @@ bool CSiteManager::LoadDefaultSites()
 	pTree->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_Expanded);
 	pTree->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_SelectedExpanded);
 
-	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites);
+	wxString lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
+	if (lastSelection[0] == '1')
+	{
+		if (lastSelection == _T("1"))
+			pTree->SelectItem(m_predefinedSites);
+		else
+			lastSelection = lastSelection.Mid(1);
+	}
+	else
+		lastSelection = _T("");
+	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites, lastSelection);
 
 	Load(pElement, &handler);
 
@@ -1867,7 +1897,13 @@ bool CSiteManager::MoveItems(wxTreeItemId source, wxTreeItemId target, bool copy
 	}
 
 	if (!copy)
+	{
+		wxTreeItemId parent = pTree->GetItemParent(source);
+		if (pTree->GetChildrenCount(parent) == 1)
+			pTree->Collapse(parent);
+
 		pTree->Delete(source);
+	}
 
 	for (std::list<wxTreeItemId>::iterator iter = expand.begin(); iter != expand.end(); iter++)
 		pTree->Expand(*iter);
@@ -1888,3 +1924,267 @@ void CSiteManager::OnChar(wxKeyEvent& event)
 	wxCommandEvent cmdEvent;
 	OnRename(cmdEvent);
 }
+
+void CSiteManager::CopyAddServer(const CServer& server)
+{
+	if (!Verify())
+		return;
+
+	AddNewSite(m_ownSites, server);
+}
+
+void CSiteManager::AddNewSite(wxTreeItemId parent, const CServer& server)
+{
+	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
+	if (!pTree)
+		return;
+
+	wxString newName = _("New site");
+	int index = 2;
+	while (true)
+	{
+		wxTreeItemId child;
+		wxTreeItemIdValue cookie;
+		child = pTree->GetFirstChild(parent, cookie);
+		bool found = false;
+		while (child.IsOk())
+		{
+			wxString name = pTree->GetItemText(child);
+			int cmp = name.CmpNoCase(newName);
+			if (!cmp)
+			{
+				found = true;
+				break;
+			}
+
+			child = pTree->GetNextChild(parent, cookie);
+		}
+		if (!found)
+			break;
+
+		newName = _("New site") + wxString::Format(_T(" %d"), index++);
+	}
+
+	wxTreeItemId newItem = pTree->AppendItem(parent, newName, 2, 2, new CSiteManagerItemData(server));
+	pTree->SortChildren(m_ownSites);
+	pTree->SelectItem(newItem);
+	pTree->Expand(m_ownSites);
+	pTree->EditLabel(newItem);
+}
+
+void CSiteManager::RememberLastSelected()
+{
+	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
+	if (!pTree)
+		return;
+
+	wxTreeItemId item = pTree->GetSelection();
+	if (!item.IsOk())
+		return;
+
+	wxString path;
+	while (item != pTree->GetRootItem() && item != m_ownSites && item != m_predefinedSites)
+	{
+		wxString name = pTree->GetItemText(item);
+		name.Replace(_T("\\"), _T("\\\\"));
+		name.Replace(_T("/"), _T("\\/"));
+
+		path = name + _T("/") + path;
+
+		item = pTree->GetItemParent(item);
+	}
+
+	if (item == m_predefinedSites)
+		path = _T("1") + path;
+	else
+		path = _T("0") + path;
+
+	COptions::Get()->SetOption(OPTION_SITEMANAGER_LASTSELECTED, path);
+}
+
+bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
+{
+	result.clear();
+
+	wxString name;
+	const wxChar *p = path;
+
+	// Undo escapement
+	bool lastBackslash = false;
+	while (*p)
+	{
+		const wxChar& c = *p;
+		if (c == '\\')
+		{
+			if (lastBackslash)
+			{
+				name += _T("\\");
+				lastBackslash = false;
+			}
+			else
+				lastBackslash = true;
+		}
+		else if (c == '/')
+		{
+			if (lastBackslash)
+			{
+				name += _T("/");
+				lastBackslash = 0;
+			}
+			else
+			{
+				if (!name.IsEmpty())
+					result.push_back(name);
+				name.clear();
+			}
+		}
+		else
+			name += *p;
+		p++;
+	}
+	if (lastBackslash)
+		return false;
+	if (name != _T(""))
+		result.push_back(name);
+
+	return !result.empty();
+}
+
+class CSiteManagerXmlHandler_ByPath : public CSiteManagerXmlHandler
+{
+public:
+	CSiteManagerXmlHandler_ByPath(const wxString& lastSelection)
+	{
+		m_theItemData = 0;
+		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection))
+			m_lastSelection.clear();
+		m_wrong_sel_depth = 0;
+	}
+
+	virtual ~CSiteManagerXmlHandler_ByPath()
+	{
+	}
+
+	virtual bool AddFolder(const wxString& name, bool expanded)
+	{
+		if (m_theItemData)
+			return false;
+
+		if (!m_wrong_sel_depth && !m_lastSelection.empty())
+		{
+			const wxString& first = m_lastSelection.front();
+			if (first == name)
+				m_lastSelection.pop_front();
+			else
+				m_wrong_sel_depth++;
+		}
+		else
+			m_wrong_sel_depth++;
+
+		return true;
+	}
+
+	virtual bool AddSite(const wxString& name, CSiteManagerItemData* data)
+	{
+		if (m_theItemData)
+		{
+			delete data;
+			return false;
+		}
+
+		if (m_wrong_sel_depth || m_lastSelection.empty())
+		{
+			delete data;
+			return true;
+		}
+
+		const wxString& first = m_lastSelection.front();
+		if (first != name)
+		{
+			delete data;
+			return true;
+		}
+
+		m_lastSelection.clear();
+		m_theItemData = data;
+
+		return true;
+	}
+
+	virtual bool LevelUp()
+	{
+		if (m_wrong_sel_depth)
+			m_wrong_sel_depth--;
+
+		return m_theItemData == 0;
+	}
+
+	// Our result
+	CSiteManagerItemData* m_theItemData;
+
+protected:
+
+	std::list<wxString> m_lastSelection;
+	int m_wrong_sel_depth;
+};
+
+CSiteManagerItemData* CSiteManager::GetSiteByPath(wxString sitePath)
+{
+	wxChar c = sitePath[0];
+	if (c != '0' && c != '1')
+	{
+		wxMessageBox(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
+		return 0;
+	}
+
+	sitePath = sitePath.Mid(1);
+
+	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
+	// to the same file or one is reading while the other one writes.
+	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
+
+	CXmlFile file;
+	TiXmlElement* pDocument = 0;
+
+	if (c == '0')
+		pDocument = file.Load(_T("sitemanager"));
+	else
+	{
+		const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
+		if (defaultsDir == _T(""))
+		{
+			wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+			return 0;
+		}
+		wxFileName name(defaultsDir, _T("fzdefaults.xml"));
+		pDocument = file.Load(name);
+	}
+
+	if (!pDocument)
+	{
+		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName().GetFullPath().c_str());
+		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+
+		return 0;
+	}
+
+	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
+	if (!pElement)
+	{
+		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+		return 0;
+	}
+
+
+	CSiteManagerXmlHandler_ByPath handler(sitePath);
+
+	Load(pElement, &handler);
+
+	if (!handler.m_theItemData)
+	{
+		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+		return 0;
+	}
+
+	return handler.m_theItemData;
+}
