@@ -8,6 +8,7 @@
 #include "filezillaapp.h"
 #include "RemoteListView.h"
 #include "recursive_operation.h"
+#include "statusbar.h"
 
 CState::CState(CMainFrame* pMainFrame)
 {
@@ -31,8 +32,11 @@ CState::~CState()
 	delete m_pEngine;
 
 	// Unregister all handlers
-	for (std::list<CStateEventHandler*>::iterator iter = m_handlers.begin(); iter != m_handlers.end(); iter++)
-		(*iter)->m_pState = 0;
+	for (int i = 0; i < STATECHANGE_MAX; i++)
+	{
+		for (std::list<CStateEventHandler*>::iterator iter = m_handlers[i].begin(); iter != m_handlers[i].end(); iter++)
+			(*iter)->m_pState = 0;
+	}
 
 	delete m_pRecursiveOperation;
 }
@@ -243,6 +247,13 @@ void CState::SetServer(const CServer* server)
 		delete m_pServer;
 	}
 
+	CStatusBar* const pStatusBar = m_pMainFrame->GetStatusBar();
+	if (pStatusBar)
+	{
+		pStatusBar->DisplayDataType(server);
+		pStatusBar->DisplayEncrypted(server);
+	}
+
 	if (server)
 		m_pServer = new CServer(*server);
 	else
@@ -251,6 +262,8 @@ void CState::SetServer(const CServer* server)
 			m_pMainFrame->SetTitle(_T("FileZilla"));
 		m_pServer = 0;
 	}
+
+	NotifyHandlers(STATECHANGE_SERVER);
 }
 
 const CServer* CState::GetServer() const
@@ -258,11 +271,6 @@ const CServer* CState::GetServer() const
 	return m_pServer;
 }
 
-void CState::ApplyCurrentFilter()
-{
-	NotifyHandlers(STATECHANGE_APPLYFILTER);
-}
-
 bool CState::Connect(const CServer& server, bool askBreak, const CServerPath& path /*=CServerPath()*/)
 {
 	if (!m_pEngine)
@@ -281,7 +289,11 @@ bool CState::Connect(const CServer& server, bool askBreak, const CServerPath& pa
 	COptions::Get()->SetLastServer(server);
 	COptions::Get()->SetOption(OPTION_LASTSERVERPATH, path.GetSafePath());
 
-	m_pMainFrame->SetTitle(server.FormatServer() + _T(" - FileZilla"));
+	const wxString& name = server.GetName();
+	if (!name.IsEmpty())
+		m_pMainFrame->SetTitle(name + _T(" - ") + server.FormatServer() + _T(" - FileZilla"));
+	else
+		m_pMainFrame->SetTitle(server.FormatServer() + _T(" - FileZilla"));
 
 	return true;
 }
@@ -308,61 +320,82 @@ void CState::DestroyEngine()
 	m_pEngine = 0;
 }
 
-void CState::RegisterHandler(CStateEventHandler* pHandler)
+void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
 {
 	wxASSERT(pHandler);
+	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 
+	std::list<CStateEventHandler*> &handlers = m_handlers[notification];
 	std::list<CStateEventHandler*>::const_iterator iter;
-	for (iter = m_handlers.begin(); iter != m_handlers.end(); iter++)
-		if (*iter == pHandler)
-			break;
-
-	if (iter != m_handlers.end())
-		return;
-
-	m_handlers.push_back(pHandler);
-}
-
-void CState::UnregisterHandler(CStateEventHandler* pHandler)
-{
-	for (std::list<CStateEventHandler*>::iterator iter = m_handlers.begin(); iter != m_handlers.end(); iter++)
+	for (iter = handlers.begin(); iter != handlers.end(); iter++)
 	{
 		if (*iter == pHandler)
-		{
-			m_handlers.erase(iter);
 			return;
+	}
+
+	handlers.push_back(pHandler);
+}
+
+void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+{
+	wxASSERT(pHandler);
+	wxASSERT(notification != STATECHANGE_MAX);
+
+	if (notification == STATECHANGE_NONE)
+	{
+		for (int i = 0; i < STATECHANGE_MAX; i++)
+		{
+			std::list<CStateEventHandler*> &handlers = m_handlers[i];
+			for (std::list<CStateEventHandler*>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+			{
+				if (*iter == pHandler)
+				{
+					handlers.erase(iter);
+					break;
+				}
+			}
+		}
+	}
+	else
+	{
+		std::list<CStateEventHandler*> &handlers = m_handlers[notification];
+		for (std::list<CStateEventHandler*>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		{
+			if (*iter == pHandler)
+			{
+				handlers.erase(iter);
+				return;
+			}
 		}
 	}
 }
 
-void CState::NotifyHandlers(unsigned int event, const wxString& data /*=_T("")*/)
+void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/)
 {
-	for (std::list<CStateEventHandler*>::iterator iter = m_handlers.begin(); iter != m_handlers.end(); iter++)
+	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
+
+	const std::list<CStateEventHandler*> &handlers = m_handlers[notification];
+	for (std::list<CStateEventHandler*>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
 	{
-		if ((*iter)->m_eventMask & event)
-			(*iter)->OnStateChange(event, data);
+		(*iter)->OnStateChange(notification, data);
 	}
 }
 
-CStateEventHandler::CStateEventHandler(CState* pState, unsigned int eventMask)
+CStateEventHandler::CStateEventHandler(CState* pState)
 {
 	wxASSERT(pState);
-	wxASSERT(eventMask);
 
 	if (!pState)
 		return;
 
 	m_pState = pState;
-	m_eventMask = eventMask;
-
-	m_pState->RegisterHandler(this);
 }
 
 CStateEventHandler::~CStateEventHandler()
 {
 	if (!m_pState)
 		return;
-	m_pState->UnregisterHandler(this);
+	m_pState->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
@@ -396,14 +429,16 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		}
 		else if (wxDir::Exists(files[i]))
 		{
-			wxString name = files[i];
-			int pos = name.Find(wxFileName::GetPathSeparator(), true);
-			if (pos != -1)
+			wxString dir = files[i];
+			if (dir.Last() == wxFileName::GetPathSeparator() && dir.Len() > 1)
+				dir.RemoveLast();
+			int pos = dir.Find(wxFileName::GetPathSeparator(), true);
+			if (pos != -1 && pos != (int)dir.Len() - 1)
 			{
-				name = name.Mid(pos + 1);
+				wxString lastSegment = dir.Mid(pos + 1);
 				CServerPath target = path;
-				target.AddSegment(name);
-				m_pMainFrame->GetQueue()->QueueFolder(queueOnly, false, files[i], target, *m_pServer);
+				target.AddSegment(lastSegment);
+				m_pMainFrame->GetQueue()->QueueFolder(queueOnly, false, dir, target, *m_pServer);
 			}
 		}
 	}
