@@ -30,10 +30,9 @@ CStatusLineCtrl::CStatusLineCtrl(CQueueView* pParent, const t_EngineData* const
 	SetOwnFont(pParent->GetFont());
 	SetBackgroundStyle(wxBG_STYLE_CUSTOM);
 	SetBackgroundColour(pParent->GetBackgroundColour());
-	
+
 	m_transferStatusTimer.SetOwner(this, TRANSFERSTATUS_TIMER_ID);
 
-	m_madeProgress = false;
 	m_pParent = pParent;
 	m_pStatus = 0;
 	m_lastOffset = -1;
@@ -115,6 +114,7 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent& event)
 	dc.SetFont(GetFont());
 	dc.SetPen(GetBackgroundColour());
 	dc.SetBrush(GetBackgroundColour());
+	dc.SetTextForeground(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
 
 	// Get character height so that we can center the text vertically.
 	wxCoord h = (rect.GetHeight() - m_textHeight) / 2;
@@ -211,9 +211,6 @@ void CStatusLineCtrl::SetTransferStatus(const CTransferStatus* pStatus)
 
 		m_lastOffset = pStatus->currentOffset;
 
-		if (!m_madeProgress && !pStatus->list && pStatus->currentOffset - pStatus->startOffset > 16384)
-			m_madeProgress = true;
-
 		if (!m_transferStatusTimer.IsRunning())
 			m_transferStatusTimer.Start(100);
 	}
@@ -234,7 +231,15 @@ void CStatusLineCtrl::OnTimer(wxTimerEvent& event)
 	if (!m_pEngineData->pEngine->GetTransferStatus(status, changed))
 		SetTransferStatus(0);
 	else if (changed)
+	{
+		if (status.madeProgress && !status.list &&
+			m_pEngineData->pItem->GetType() == QueueItemType_File)
+		{
+			CFileItem* pItem = (CFileItem*)m_pEngineData->pItem;
+			pItem->m_madeProgress = true;
+		}
 		SetTransferStatus(&status);
+	}
 	else
 		m_transferStatusTimer.Stop();
 }
@@ -270,7 +275,7 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height)
 	dc.DrawRectangle(x + 1 + barSplit, y + 1, PROGRESSBAR_WIDTH - barSplit - 1, height - 2);
 
 	if (barSplit && height > 2)
-	{	
+	{
 		// Draw pretty gradient
 
 		int greenmin = 128;
