@@ -81,6 +81,7 @@ CViewHeader::CViewHeader(wxWindow* pParent, const wxString& label)
 {
 	m_alreadyInPaint = false;
 	m_pComboBox = new CComboBoxEx(this);
+	m_pLabel = new wxStaticText(this, wxID_ANY, label, wxDefaultPosition, wxDefaultSize);
 	wxSize size = GetSize();
 #ifdef __WXMSW__
 	size.SetHeight(m_pComboBox->GetBestSize().GetHeight());
@@ -95,6 +96,7 @@ CViewHeader::CViewHeader(wxWindow* pParent, const wxString& label)
 	m_pComboBox->Connect(wxID_ANY, wxEVT_LEFT_UP, (wxObjectEventFunction)(wxEventFunction)(wxMouseEventFunction)&CViewHeader::OnComboMouseEvent, 0, this);
 	m_bLeftMousePressed = false;
 #endif //__WXMSW__
+
 	SetLabel(label);
 }
 
@@ -109,6 +111,13 @@ void CViewHeader::OnSize(wxSizeEvent& event)
 	rect.SetWidth(rect.GetWidth() - 5);
 #endif
 	m_pComboBox->SetSize(rect);
+
+	rect.SetX(5);
+	rect.SetWidth(m_cbOffset - 5);
+	rect.SetY((rect.GetHeight() - m_labelHeight) / 2 - 1);
+	rect.SetHeight(m_labelHeight);
+	m_pLabel->SetSize(rect);
+
 	Refresh();
 }
 
@@ -231,20 +240,18 @@ void CViewHeader::OnPaint(wxPaintEvent& event)
 	wxRect rect = GetClientRect();
 	wxPaintDC dc(this);
 	dc.SetPen(*wxBLACK_PEN);
+	dc.SetTextForeground(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
 
 #ifdef __WXMSW__
 	dc.DrawLine(rect.GetLeft(), rect.GetBottom(), m_cbOffset, rect.GetBottom());
 #else
 	dc.DrawLine(rect.GetLeft(), rect.GetBottom(), rect.GetRight(), rect.GetBottom());
 #endif
-
-	dc.SetFont(GetFont());
-	dc.DrawText(m_label, 5, (rect.GetHeight() - m_labelHeight) / 2 - 1);
 }
 
 void CViewHeader::SetLabel(const wxString& label)
 {
-	m_label = label;
+	m_pLabel->SetLabel(label);
 	int w;
 	GetTextExtent(label, &w, &m_labelHeight);
 	m_cbOffset = w + 10;
@@ -262,7 +269,7 @@ void CViewHeader::Reparent(CViewHeader** pViewHeader, wxWindow* parent)
 
 wxString CViewHeader::GetLabel() const
 {
-	return m_label;
+	return m_pLabel->GetLabel();
 }
 
 void CViewHeader::AddRecentDirectory(const wxString &directory)
@@ -318,8 +325,9 @@ EVT_COMMAND(wxID_ANY, fzEVT_LOCALVIEWHEADERSETTEXTSEL, CLocalViewHeader::OnSelec
 END_EVENT_TABLE()
 
 CLocalViewHeader::CLocalViewHeader(wxWindow* pParent, CState* pState)
-	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(pState, STATECHANGE_LOCAL_DIR)
+	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(pState)
 {
+	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 }
 
 void CLocalViewHeader::OnTextChanged(wxCommandEvent& event)
@@ -497,10 +505,9 @@ void CLocalViewHeader::OnTextEnter(wxCommandEvent& event)
 	}
 }
 
-void CLocalViewHeader::OnStateChange(unsigned int event, const wxString& data)
+void CLocalViewHeader::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
-	if (event != STATECHANGE_LOCAL_DIR)
-		return;
+	wxASSERT(notification == STATECHANGE_LOCAL_DIR);
 
 #ifdef __WXGTK__
 	m_autoCompletionText = _T("");
@@ -518,15 +525,15 @@ EVT_COMBOBOX(wxID_ANY, CRemoteViewHeader::OnSelectionChanged)
 END_EVENT_TABLE()
 
 CRemoteViewHeader::CRemoteViewHeader(wxWindow* pParent, CState* pState)
-	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(pState, STATECHANGE_REMOTE_DIR)
+	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(pState)
 {
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	m_pComboBox->Disable();
 }
 
-void CRemoteViewHeader::OnStateChange(unsigned int event, const wxString& data)
+void CRemoteViewHeader::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
-	if (event != STATECHANGE_REMOTE_DIR)
-		return;
+	wxASSERT(notification == STATECHANGE_REMOTE_DIR);
 
 	m_path = m_pState->GetRemotePath();
 	if (m_path.IsEmpty())
