@@ -25,6 +25,7 @@ public:
 
 protected:
 	CComboBoxEx* m_pComboBox;
+	wxStaticText* m_pLabel;
 	bool m_alreadyInPaint;
 
 	DECLARE_EVENT_TABLE();
@@ -40,7 +41,6 @@ protected:
 	void AddRecentDirectory(const wxString &directory);
 	std::list<wxString> m_recentDirectories;
 
-	wxString m_label;
 	int m_cbOffset;
 	int m_labelHeight;
 };
@@ -51,7 +51,7 @@ public:
 	CLocalViewHeader(wxWindow* pParent, CState* pState);
 
 protected:
-	virtual void OnStateChange(unsigned int event, const wxString& data);
+	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
 
 	wxString m_oldValue;
 
@@ -72,7 +72,7 @@ public:
 	CRemoteViewHeader(wxWindow* pParent, CState* pState);
 
 protected:
-	virtual void OnStateChange(unsigned int event, const wxString& data);
+	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
 
 	CServer m_lastServer;
 	CServerPath m_path;
